/*====================================================================

Objectweb Browser Framework
Copyright (C) 2000-2003 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.browser.api;

/**
 * <p>
 * Defines an menu item in a popup menu. <br>
 * In order to define a menu item, you can have a look to the corresponding
 * <a href="../../../../../../resources/browser_dtd.html#menu">DTD element</a> or to 
 * an <a href="package-summary.html#xml">XML example.</a>
 * </p>
 * 
 * <p>
 * Example:
 * <pre>
 *      public int 
 *      getStatus(TreeView treeView) 
 *      {
 *          // Enabled item
 *          return MenuItem.ENABLED_STATUS; 
 *      }
 *
 *      public void 
 *      actionPerformed(MenuItemTreeView treeView) 
 *      throws Exception 
 *      {
 *          // Obtaining the name of the entry
 *          String value = (String)treeView.getSelectedEntry().getName();
 *          // Obtaining the Java Properties
 *          Properties properties = (Properties)treeView.getParentObject();
 *          // Removing the property
 *          properties.remove(value);
 *      }
 * </pre>
 * This action allows to remove a property from a <code>java.util.Properties<code> object.
 * It consists of obtaining the name of the entry, obtaining the <code>java.util.Properties<code> object
 * and removing the property entry. <br>
 * This action is defined for <code>java.util.Properties<code> object but is available 
 * at the tree child level, because of the <code>tree-child-visible="true"</code> tag in the XML document. 
 * </p>
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>,
 *         <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>
 *         
 * @version 0.2
 */
public interface MenuItem 
{

    /** Indicates that a menu is enabled */
    public static final int ENABLED_STATUS = 1;

    /** Indicates that a menu is disabled */
    public static final int DISABLED_STATUS = ENABLED_STATUS + 1;

    /** Indicates that a menu is not visible */
    public static final int NOT_VISIBLE_STATUS = DISABLED_STATUS + 1;

    /**
     * Indicates if the menu item must be enabled, disabled or not visible.
     * <br>Possible values:
     * <ul>
     * <li>ENABLED_STATUS: The menu item is enabled.</li>
     * <li>DISABLED_STATUS: The menu item is disabled.</li>
     * <li>NOT_VISIBLE_STATUS: The menu item is not visible.</li>
     * </ul>
     *  
     * @param treeView A view to obtain the node to check.
     * 
     * @return The status associated to the menu item.
     */
    public int 
    getStatus(TreeView treeView);

    /**
     * Invoked when an action occurs on the menu item.
     *
     * @param treeView The contextual MenuItemTreeView.
     * 
     * @exception Exception The invocation may thrown every kind of exception.
     */
    public void 
    actionPerformed(MenuItemTreeView treeView) 
    throws Exception;
    
}
