/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.api;

/**
 * Names are non-mutable structures representing names used by contexts.
 *
 * A Name is a pair <id,subname>,
 * where id is an identifier represented as a String,
 * and subname a (possibly null) Name.
 *
 * The "" identifier is reserved in contexts representing trees
 * to represent the root of the tree.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 *
 * Note: This is strongly inspired from the Jonathan naming context framework
 * initially developped by Fabien Delpiano and Bruno Dumant.
 */
public interface Name {

    /**
     * The standard name of this interface to use
     * in the context of the Fractal framework.
     */
    static final String NAME = Name.class.getName();

    /**
     * Gets the id of the target name.
     *
     * This method does not update the id of the target name.
     *
     * @return The id of the target name.
     *
     * @postcondition return == getOWId();
     */
    String getOWId();

    /**
     * Gets the subname of the target name.
     *
     * This method does not update the subname of the target name.
     *
     * @return the subname of the target name.
     *
     * @postcondition return == getOWSubName();
     */
    Name getOWSubName();

    /**
     * Tests if a given name is equals to the target name.
     *
     * @param name The given name.
     *
     * @return
     *  true if the given name is the same object of the target name,
     *     or if the id and the subname of the given name are equals to
     *            respectively the id and subname of the target name;
     *  false otherwise.
     *
     * @postcondition
     *     result == (name == this)
     *  or (          name.getOWId().equals(this.getOWId())
     *       and name.getOWSubName().equals(this.getOWSubName()) );
     */
    boolean equals(Name name);

}
