/*===========================================================================

ObjectWeb Naming Context Framework
Copyright (C) 2002 USTL - LIFL - GOAL
Contact: architecture@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s):  ______________________________________.

===========================================================================*/

package org.objectweb.util.browser.core.naming;

/** To implement the Name interface. */
import org.objectweb.util.browser.core.api.Name;

/**
 * Abstract basic implementation of the Name interface which
 * only provides an base implementation for the toStringBuffer and equals methods.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 * @version 0.1
 */
public abstract class AbstractName extends AbstractObject implements Name {

    // ==================================================================
    //
    // No internal state.
    //
    // ==================================================================

    // ==================================================================
    //
    // No constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // No internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods for class Object.
    //
    // ==================================================================

    /**
     * Tests if a given object is equals to the target name.
     *
     * @param obj The given object.
     *
     * @return
     *  true if the given object is a Name instance and
     *          this given name is equals to the target name;
     *  false otherwise.
     *
     * @postcondition
     *     obj instanceof Name
     *  and result == equals((Name)obj);
     */
    public boolean equals(Object obj) {
        if (obj instanceof Name)
            return equals((Name) obj);
        else
            return super.equals(obj);
    }

    // ==================================================================
    //
    // Public methods for class AbstractObject.
    //
    // ==================================================================

    /**
     * Appends internal state representation into a given string buffer.
     *
     * This method does not update the target object.
     *
     * @param sb The string buffer where pairs <name,value> are appended.
     *
     * @postcondition isNotModified();
     */
    public void toStringBuffer(StringBuffer sb) {
        sb.append("id=");
        sb.append(getOWId());
        sb.append(",subname=");
        sb.append(getOWSubName());
    }
    
    // ==================================================================
    //
    // Public methods for interface Name.
    //
    // ==================================================================

    /**
     * Gets the id of the target name.
     *
     * This method does not update the id of the target name.
     *
     * @return The id of the target name.
     *
     * @postcondition return == getOWId();
     */
    public abstract String getOWId();

    /**
     * Gets the subname of the target name.
     *
     * This method does not update the subname of the target name.
     *
     * @return the subname of the target name.
     *
     * @postcondition return == getOWSubName();
     */
    public abstract Name getOWSubName();

    /**
     * Tests if a given name is equals to the target name.
     *
     * @param name The given name.
     *
     * @return
     *  true if the given name is the same object of the target name,
     *     or if the id and the subname of the given name are equals to
     *           respectively the id and subname of the target name;
     *  false otherwise.
     *
     * @postcondition
     *     result == (name == this)
     *  or (          name.getOWId().equals(this.getOWId())
     *       and name.getOWSubName().equals(getOWSubName()) );
     */
    public boolean equals(Name name) {
        return (name == this) || (name.getOWId().equals(this.getOWId()) && name.getOWSubName().equals(getOWSubName()));
    }

}
