/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.openccm.corba.idl2java;

// Package dependencies.
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

/**
 * Application to wrap an IDL to Java generator.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */
public abstract class IdlToJavaApplication
              extends IdlToJavaApplicationBase
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    /** The IDL file to parse. */
    protected String input_file_;
    
    /** The destination directory for generated files. */
    protected String dest_dir_; 
    
    /** List of directories to include in the search path. */
    protected List includes_;
    
    /** Package mappings definitions. */
    protected Map package_mappings_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================
    
    /**
     * The default constructor.
     */
    public 
    IdlToJavaApplication()
    {
        // Call the IdlToJavaApplicationBase constructor
        super();
        
        // Init internal states
        input_file_ = null;
        dest_dir_ = null;
        includes_ = new java.util.ArrayList();
        package_mappings_ = new java.util.HashMap();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.util.cmdline.api.Application
    //
    // ==================================================================

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.corba.idl2java.IdlToJavaOperations
    //
    // ==================================================================

    /**
     * Set the IDL file to parse.
     * 
     * @param file - The input file.
     */
    public void
    setInputFile(String file)
    {
        input_file_ = file;
    }
    
    /**
     * Get the IDL file to parse.
     * 
     * @return The input file.
     */
    public String
    getInputFile()
    {
        return input_file_;
    }
    
    /**
     * Set the destination directory for generated java files.
     * 
     * @param destdir - The destination directory.
     */
    public void
    setDestinationDirectory(String destdir)
    {
        dest_dir_ = destdir;
    }
    
    /**
     * Get the destination directory for generated java files.
     * 
     * @return The destination directory.
     */
    public String
    getDestinationDirectory()
    {
        return dest_dir_;
    }
    
    /**
     * Add a directory to the include list.
     * (Preprocessor -I option)
     * 
     * @param directory - The directory to add.
     */
    public void
    addIncludeDirectory(String directory)
    {
        includes_.add(directory);
    }

    /**
     * Get directories added to the include list.
     * (Preprocessor -I option)
     * 
     * @return The list of included directories.
     */
    public String[]
    getIncludeDirectories()
    {
        return (String[]) includes_.toArray(new String[0]);
    }

    /**
     * Add a package mapping definition.
     * 
     * @param idl_package - The idl package name.
     * @param java_package - The java package name.
     */
    public void
    addPackageMapping(String idl_package, String java_package)
    {
        package_mappings_.put(idl_package, java_package);
    }

    /**
     * Get package mapping definitions.
     * 
     * @return The map of package mappings.
     */
    public Map
    getPackageMappings()
    {
        return package_mappings_;
    }

    // ==================================================================
    //
    // Public methods
    //
    // ==================================================================

    /**
      * Parse and store command line options.
      *
      * @param args The command line arguments.
      */
    public void
    parse_cmdline(String[] args)
    {
        String[] mappings = null;
        StringTokenizer st = null;
        
        dest_dir_ = destdir_opt_.getArgument();
        input_file_ = args[0];
        
        mappings = i2jpackage_opt_.getOptionValues();
        for (int i=0; i<mappings.length; i++)
        {
            st = new StringTokenizer(mappings[i], ":");
            try
            {
                addPackageMapping(st.nextToken(), st.nextToken());
            } catch (NoSuchElementException ex) {
                getConsole().error("Warning : No mapping defined for " + mappings[i]);
            }
        }
    }
}
