/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Sylvain Leblanc
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.corba.trader;

// Package dependencies
import org.omg.CosTrading.FollowOption;

/**
 * Provides an helper class in order to deal with standard trader
 * querying policies. Mainly allows to check if a given policy exist
 * and to retrieve the type code associated with this policy.
 *
 * @author <a href="mailto:Sylvain.Leblanc@lifl.fr">Sylvain Leblanc</a>
 * @version 0.1
 */
public abstract class QueryPoliciesHelper {

    /** 
     * The policies supported by the query operation. This map
     * contains the name of policies as keys and their related
     * org.omg.CORBA.TypeCode as value.
     */
    private static java.util.Map query_policies = null;

    // Static block used to initialize the query_policies map.
    static {
        java.util.HashMap hm = new java.util.HashMap();
        org.omg.CORBA.TypeCode ul_tc = TraderTypeCodeUtils.getTypeCode("unsigned long");
        org.omg.CORBA.TypeCode bool_tc = TraderTypeCodeUtils.getTypeCode("boolean");
        org.omg.CORBA.TypeCode fo_tc = org.omg.CosTrading.FollowOptionHelper.type();
        org.omg.CORBA.TypeCode tn_tc = org.omg.CosTrading.TraderNameHelper.type();
        org.omg.CORBA.TypeCode os_tc = org.omg.CosTrading.AdminPackage.OctetSeqHelper.type();
        
        hm.put("search_card", ul_tc);
        hm.put("match_card", ul_tc);
        hm.put("return_card", ul_tc);
        hm.put("hop_count", ul_tc);
        hm.put("link_follow_rule", fo_tc);
        hm.put("starting_trader", tn_tc);
        hm.put("request_id", os_tc);
        hm.put("exact_type_match", bool_tc);
        hm.put("use_modifiable_properties", bool_tc);
        hm.put("use_dynamic_properties", bool_tc);
        hm.put("use_proxy_offers", bool_tc);
        QueryPoliciesHelper.query_policies = hm;
    }

    /**
     * Resolve the type code of a trading query policy.
     *
     * @param policy_name the policy name to resolve.
     *
     * @return The associated CORBA type code.
     */
    public static org.omg.CORBA.TypeCode 
    getQueryPolicyType(String policy_name) 
    {
        return (org.omg.CORBA.TypeCode)QueryPoliciesHelper.query_policies.get(policy_name.toLowerCase());
    }

    /**
     * Resolve the type code of a trading query policy.
     *
     * @param policy_name the policy name to resolve.
     *
     * @return The associated CORBA type code.
     */
    public static boolean 
    exist(String policy_name) 
    {
        return QueryPoliciesHelper.query_policies.containsKey(policy_name.toLowerCase());
    }
    
    /**
     * Construct a standard CosTrading::Policy. As the given value is
     * not associated to a type code, this method assumes that the
     * policy must be an importer policy as specified in the Trading
     * Object Service specification. Otherwise, the
     * CosTrading::Property cannot be constructed and the
     * <code>null</code> value is returned.
     *
     * @param name The name of the trader policy.
     * @param value The value of the trader policy as a string.
     *
     * @return The CosTrading::Policy corresponding structure or null
     * if this policy cannot be constructed.
     */
    public static org.omg.CosTrading.Policy 
    construct_policy(String name, String value)
    {
        org.omg.CORBA.TypeCode tc = QueryPoliciesHelper.getQueryPolicyType(name);
        if (tc == null) return null;
        org.omg.CORBA.Any any = null;
        if (name.equals("link_follow_rule")) {
            any = org.objectweb.openccm.corba.TheORB.getORB().create_any();
            FollowOption opt = null;
            if (value.equals("local_only")) opt = FollowOption.local_only;
            else if (value.equals("if_no_local")) opt = FollowOption.if_no_local;
            else opt = FollowOption.always;
            org.omg.CosTrading.FollowOptionHelper.insert(any, opt);
        } else {
            any = TraderTypeCodeUtils.insertInAny(tc, value);
        }
        if (any == null) return null;
        return new org.omg.CosTrading.Policy(name, any);
    }
}
