/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2005 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): Christophe Contreras, Philippe Merle.

====================================================================
$Id: LauncherApplication.java,v 1.9 2005/06/01 18:42:49 merle Exp $
====================================================================*/

package org.objectweb.openccm.task;

//Package dependencies
import java.io.File;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import org.apache.tools.ant.BuildException;

/**
 * This Class is an abstraction for Launcher configuration.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public abstract class LauncherApplication
{
    // ==================================================================
    //
    // Internal states.
    //
    // ==================================================================

    /** Properties of the Ant Project. */
    protected Properties project_properties_;

    /** List of properties to add to the JVM. */
    private Properties properties_;

    /** Pathes to add in the classpath. */
    private List pathes_;

    /** Application arguments. */
    private List arguments_;

    /** The launcher xml configuration file. */
    private String xml_file_;

    /** The run_id to launch. */
    private String run_id_;

    /** The Launcher Application Process. */
    private Process process_;

    /** Arguments delimiter. */
    private String arg_delimiter_;

    /** Launcher working directory. */
    private File working_directory_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public LauncherApplication()
    {
        project_properties_ = new Properties();
        properties_ = new Properties();
        pathes_ = new ArrayList();
        arguments_ = new ArrayList();
        run_id_ = null;
        xml_file_ = null;
        process_ = null;
        if ( System.getProperty("file.separator").compareTo("\\") == 0 )
            arg_delimiter_ = "\"";
        else
            arg_delimiter_ = "";
        working_directory_ = null;
    }
    
    /**
     * The constructor with project properties parameter.
     * 
     * @param project_properties - Properties of the Ant Project.
     */
    public LauncherApplication(Map project_properties)
    {
        this();
        project_properties_.putAll(project_properties); 
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Get the JDK to run a 'java' command.
     * 
     * @return The full path to the java command.
     */
    private String
    getJDK()
    {
        return System.getProperty("java.home") + "/bin/java";
    }
    
    /**
     * Get the JDK bootclasspath.
     * 
     * @return A bootclasspath declaration string.
     */
    private String
    getBootClasspath()
    {
        String cp = null;
        
        if (project_properties_.getProperty("CORBA.launcher.bootclasspath")
            != null)
            cp = project_properties_.getProperty("CORBA.launcher.bootclasspath");
        if ( cp == null)
            return "";
        else
            return "-Xbootclasspath/p:"
                + cp ;
    }
    
    /**
     * Get the JDK properties declaration.
     * 
     * @return A properties declaration string.
     */
    private String
    getProperties()
    {
        String props = "";
        
        for (Enumeration e = properties_.keys() ; e.hasMoreElements() ;)
        {
            String key = (String) e.nextElement();
            props += arg_delimiter_+ "-D" + key + "=" + properties_.getProperty(key) + arg_delimiter_ + " "; 
        }

        /**
         * Retrieves any OpenCCM_* and ORB_HOMEDIR property to propagate it to
         *  the Java System Properties of the launched application.
         *
         * [ analog to retrieving the shell environment variables and
         *   passing them to the Launcher with -D , as done in the
         *   openccm_launcher scripts ]
         */
        for (
            Enumeration openccm_envi = project_properties_.propertyNames();
            openccm_envi.hasMoreElements();
            )
        {
            String key = (String) openccm_envi.nextElement();
            if (key.startsWith("OpenCCM_") || key.equals("ORB_HOMEDIR"))
                props += arg_delimiter_+ "-D" + key + "=" + project_properties_.getProperty(key) + arg_delimiter_ + " ";
        }
        /** */

        return props;
    }
    
    /**
     * Get the Launcher classpath declaration.
     * 
     * @return The Launcher classpath declaration string.
     */
    private String
    getClasspath()
    {
        String classpath = "-classpath " + arg_delimiter_;
        String separator = System.getProperty("path.separator");

        // Add pathes for Launcher
        if (project_properties_.getProperty("CORBA.launcher.classpath")
            !=null)
            classpath += project_properties_.getProperty("CORBA.launcher.classpath")+ separator;

        // Add user pathes
        for (Iterator it = pathes_.iterator(); it.hasNext();)
        {
            classpath += it.next() + separator; 
        }
        
        // Remove the last separator
        return classpath.substring(0, classpath.length()-1) + arg_delimiter_;
    }
    
    /**
     * Get Launcher arguments declaration.
     * 
     * @return Launcher arguments declaration string.
     */
    private String
    getArguments()
    {
        String args = "";
        
        for (Iterator it = arguments_.iterator(); it.hasNext();)
        {
            args += arg_delimiter_ + it.next() + arg_delimiter_ + " "; 
        }
        
        return args;
    }
    
    /**
     * Get Launcher run_id declaration.
     * 
     * @return Launcher run_id declaration string.
     */
    private String
    getRunId()
    {
        if (run_id_ != null)
            return "--runid " + run_id_;
        else
            return "";
    }
    
    /**
     * Get the Launcher xml configuration file.
     * 
     * @return The Launcher xml configuration file.
     */
    private String
    getXmlFile()
    {
        return xml_file_;
    }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Set ant project properties.
     * 
     * @param project_properties - List of project properties.
     */
    public void
    setProjectProperties(Map project_properties)
    {
        project_properties_.putAll(project_properties);
    }
    
    /**
     * Set the Working directory.
     * 
     * @param dir - A directory.
     */
    public void
    setWorkingDirectory(File dir)
    {
        working_directory_ = dir; 
    }
    
    /**
     * Add a property to Launcher.
     * 
     * @param name - The property name.
     * @param value - The property value.
     */
    public void
    addProperty(String name, String value)
    {
        properties_.setProperty(name, value);
    }

    /**
     * Add a path to Launcher ClassLoader.
     * 
     * @param path - The path to add.
     */
    public void
    addPath(String path)
    {
        pathes_.add(path);
    }

    /**
     * Add an argument to the application.
     * 
     * @param arg - The argument to add.
     */
    public void
    addArgument(String arg)
    {
        arguments_.add(arg);
    }

    /**
     * Set the Launcher xml configuration file.
     * 
     * @param file - The xml configuration file.
     */
    public void
    setXmlFile(String file)
    {
        this.xml_file_ = file;
    }

    /**
     * Set the run_id to launch.
     * 
     * @param run_id - the run_id to launch.
     */
    public void
    setRunId(String run_id)
    {
        this.run_id_ = run_id;
    }

    /**
     * Get the command line to execute the launcher
     * with its configuration.
     * 
     * @return The ommand line to execute.
     */
    public String
    getCommandLine()
    {
        String result = null;

        result = getJDK() + " ";
        result += getBootClasspath() + " ";
        result += getProperties() + " ";
        result += getClasspath() + " ";
        result += "org.objectweb.util.launcher.Launcher ";
		result += arg_delimiter_ + "file:" + getXmlFile() + arg_delimiter_ + " ";
		result += getRunId() + " ";
        result += getArguments();
              
        return result;
    }

    /**
     * Configure the application.
     * Sub-classes must implement this method.
     */
    abstract public void
    configure();

    /**
     * Callback method to insert code after
     * application launched.
     */
    public void
    post_exec() {}

    /**
     * Run the launcher application as an external command
     * and wait for process termination.
     * 
     * @throws BuildException if an error occurs.
     */
    public void
    run()
    throws BuildException
    {
        run(false);
    }

    /**
     * Run the launcher application as an external command.
     * 
     * @param standalone - If true the process is run in a standalone mode
     *                     else, this method waits for process termination.
     * 
     * @throws BuildException if an error occurs.
     */
    public void
    run(boolean standalone)
    throws BuildException
    {
        Redirector redirector = null;

        configure();
        try
        {
            // System.out.println("exec="+ getCommandLine());
            process_ = Runtime.getRuntime().exec( getCommandLine(), null, working_directory_ );
            
            redirector = new Redirector(process_);
            redirector.redirectOutput();

            post_exec();
        }catch(Exception e){
            e.printStackTrace();
        }

        if (!standalone)
        {
            try
            {
                process_.waitFor();
            }catch (InterruptedException e) {
                e.printStackTrace();
            }
            if (process_.exitValue() == -1)
            {
                throw new BuildException("OpenCCM build failed!");
            }
        }
    }
    
    /**
     * Stop the launcher application process.
     */
    public void
    stop()
    {
        if (process_ != null)
        {
            process_.destroy();
        }
    }
}
