/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Briclet Frederic
Contributor(s): ___________________________________________________.

====================================================================*/


package org.objectweb.openccm.descriptor;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.LinkedList;
import java.util.zip.*;
import org.objectweb.openccm.descriptor.StreamManager;

/**
 * The goal of thhis class is to provide facilities to
 * retrieve a file into a zip file.
 */
public class ZipEntryRetriever {
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================
    
    //The main zip file
    private ZipFile zf;
    //The temporaryfile used to load the zipfile
    private File tmpFile;
    //The translation table to convert file path
    private java.util.Hashtable table;
    
    private StreamManager streamManager;
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Internal method to setup the convert table
     */
    private void 
    setIdentifierTable()
    {
      String entryName;
      
      for(java.util.Enumeration entries=zf.entries();
            entries.hasMoreElements();)
        {
            entryName=((java.util.zip.ZipEntry)entries.nextElement()).getName();               
            table.put(entryName.replace('\\','/').trim(),entryName);                
        }
                
    }    
    
    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================
    
    /**
     * Main constructor taking in prameter the zip file
     * to wrap
     * @param zf The zip file to wrap
     */
    public ZipEntryRetriever(ZipFile zf){
     this.zf=zf;   
     table=new java.util.Hashtable();
     setIdentifierTable();
    }
    
    /**
     * 
     * Build a new ZipEntryRetriever from a InputStream. The given 
     * inputStream will be entirely readed and buffered during initialization. 
     * 
     * @param in The inputStream to used to build the retriever
     * @return
     * @throws java.util.zip.ZipException
     * @throws java.io.IOException
     */
    public 
    ZipEntryRetriever(InputStream in)
    throws Exception
    {
        tmpFile=File.createTempFile("componentarchive"+System.currentTimeMillis(),".car");
        FileOutputStream out=new FileOutputStream(tmpFile);
        byte [] byteArray= new byte[32000];
        
        for(int readed=in.read(byteArray,0,32000); readed>0;
            readed=in.read(byteArray,0,32000))
            out.write(byteArray,0,readed);                
        
        out.flush();           
        out.close();
        zf=new ZipFile(tmpFile); 
        table=new java.util.Hashtable();
        setIdentifierTable();
        //tmpFile.deleteOnExit();
    }
    /**
     * 
     * Build a new ZipEntryRetriever from a File already write
     * on the current file system.
     * 
     * @param in The File to used to build the retriever
     * @return
     * @throws java.util.zip.ZipException
     * @throws java.io.IOException
     */
    public 
    ZipEntryRetriever(File file)
    throws Exception
    {
        tmpFile=file;
        zf=new ZipFile(tmpFile); 
     //   getStreamManager().connectOpenZipFile(zf);
        table=new java.util.Hashtable();
        setIdentifierTable();
    }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================
    
    /**
     * Return the ZipEntry denote by the given entryName. Directory delimiters
     * can indefferently be '\' or '/' or a mix of both. In all case the entryName
     * is translate in a internal representation to warrant the file resolution.
     * @param entryName the entry file name.
     */
    public ZipEntry
    retrieveZipEntryIgnoringDelimiterType(String entryName)
    throws java.util.zip.ZipException, java.io.IOException
    {
       if(!table.containsKey(entryName.replace('\\','/').trim()))
        throw new java.util.zip.ZipException("Cannot found zip entry:"+entryName) ;
             
       String originalName=(String)table.get(entryName.replace('\\','/'));       
       return zf.getEntry(originalName);
    }
    
    /**
     * This method return a list of entries that ended by the given
     * @param ending.
     * 
     * @param ending the suffix search in the entry
     * @return
     */
    public String[]
    getZipEntriesEndingBy(String ending)
    {
        LinkedList ll=new LinkedList();
         for(java.util.Enumeration entries=zf.entries();
               entries.hasMoreElements();)
           {
               ZipEntry ze=(java.util.zip.ZipEntry)entries.nextElement();
               if(ze.getName().endsWith(ending))
                ll.add(ze.getName());    
           }    
       return (String[])ll.toArray(new String[ll.size()]);        
    }
    
    /**
     * Return an InputStream open on the file denoted by the entryName. 
     * retrieveZipEntryIgnoringDelimiterType(String entryName) is used 
     * to found the zip entry.
     * @param entryName the entry file name.
     */
    public java.io.InputStream
    getZipEntryISIgnoringDelimiterType(String entryName)
    throws java.util.zip.ZipException,java.io.IOException
    {
       java.io.InputStream is= zf.getInputStream(retrieveZipEntryIgnoringDelimiterType(entryName));
       getStreamManager().connectOpenInputStream(is);
       return is;     
    }
    
    /**
     * This method open a new stream on the current zip file to allow a distant 
     * downloading.
     * 
     * @return a new input stream on the current zip file
     * @throws IOException is thrown if a problem occur using file.
     */
    public InputStream
    getNewInputStream()
    throws IOException
    {
        InputStream is=
            new FileInputStream(tmpFile);
        getStreamManager().connectOpenInputStream(is);
        return is;
    }
    
    /**
     * Connect the default stream manager in charge to close
     * all the open file and stream use by this zipEntryRetriever
     */
    public void
    connectStreamManager(StreamManager streamManager)
    {
        this.streamManager=streamManager;
        getStreamManager().connectOpenZipFile(zf);
        if(tmpFile!=null)
            getStreamManager().connectTemporaryFile(tmpFile);
    }
    
    /**
     * Return the current stream manager
     */
     public StreamManager
     getStreamManager()
     {
         return streamManager;
     }
}
