/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.parser.lib;

import java.util.Iterator;
import java.util.List;

import org.objectweb.util.explorer.core.common.api.ExplorerConstants;
import org.objectweb.util.explorer.core.menu.api.AcceleratorDescription;
import org.objectweb.util.explorer.core.menu.api.ItemDescription;
import org.objectweb.util.explorer.core.menu.api.MenuDescription;
import org.objectweb.util.explorer.core.menu.api.MnemonicDescription;
import org.objectweb.util.explorer.core.menu.lib.BasicAcceleratorDescription;
import org.objectweb.util.explorer.core.menu.lib.BasicItemDescription;
import org.objectweb.util.explorer.core.menu.lib.BasicMenuDescription;
import org.objectweb.util.explorer.core.menu.lib.BasicMnemonicDescription;
import org.objectweb.util.explorer.explorerConfig.Accelerator;
import org.objectweb.util.explorer.explorerConfig.Item;
import org.objectweb.util.explorer.explorerConfig.Menu;
import org.objectweb.util.explorer.explorerConfig.Mnemonic;
import org.objectweb.util.explorer.explorerConfig.Node;

/**
 * This component manages the "menu" XML element.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class MenuManager 
     extends AbstractNodeParser 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================
    
    /**
     * Provides the <code>AcceleratorDescription</code> according to the
     * given accelerator definition.
     * @param accelerator The accelerator definition
     * @return The corresponding accelerator description.
     */
    protected AcceleratorDescription getAcceleratorDescription(Accelerator accelerator){
        AcceleratorDescription acceleratorDesc = null;
        if(accelerator!=null){
            acceleratorDesc = new BasicAcceleratorDescription();
            acceleratorDesc.setAcceleratorCharacter(accelerator.getXmlchar().charAt(0));
            acceleratorDesc.setCtrlKey(accelerator.getCtrl().equals("true"));
            acceleratorDesc.setAltKey(accelerator.getAlt().equals("true"));
            acceleratorDesc.setShiftKey(accelerator.getShift().equals("true"));
            acceleratorDesc.setMetaKey(accelerator.getMeta().equals("true"));
        }
        return acceleratorDesc;
    }
    
    /**
     * Provides the <code>MnemonicDescription</code> according to the
     * given mnemonic definition.
     * @param mnemonic The mnemonic definition
     * @return The corresponding mnemonic description.
     */
    protected MnemonicDescription getMnemonicDescription(Mnemonic mnemonic){
        MnemonicDescription mnemonicDesc = null;
        if(mnemonicDesc!=null){
            mnemonicDesc = new BasicMnemonicDescription();
            mnemonicDesc.setMnemonicCharacter(mnemonic.getXmlchar().charAt(0));
        }
        return mnemonicDesc;
    }
    
    /**
     * Provides the <code>ItemDescription</code> according to the
     * given item definition.
     * @param item The item definition
     * @return The corresponding item description.
     */
    protected ItemDescription getItemDescription(Item item){
        ItemDescription itemDesc = null;
        if(item!=null){
            itemDesc = new BasicItemDescription();
            itemDesc.setLabel(item.getLabel());
            itemDesc.setTreeChildVisible(item.getTreeChildVisible().equals("true"));
            itemDesc.setTypeChildVisible(item.getTypeChildVisible().equals("true"));
            itemDesc.setCodeDescription(ParserUtils.getCodeDescription(item.getCode()));
            itemDesc.setIconDescription(ParserUtils.getIconDescription(item.getIcon()));
            itemDesc.setMnemonicDescription(getMnemonicDescription(item.getMnemonic()));
            itemDesc.setAcceleratorDescription(getAcceleratorDescription(item.getAccelerator()));
        }
        return itemDesc;
    }
    
    /**
     * Feeds the given <code>MenuDescription</code> with the given list of items.
     * @param The list of items to add.
     * @param menuDesc The <code>MenuDescription</code> to feed.
     */
    protected void feedMenuDescription(List elements, MenuDescription menuDesc){
        if(elements!=null && !elements.isEmpty()){
            Iterator it = elements.iterator();
            while(it.hasNext()){
                menuDesc.addMenuElement(getItemDescription((Item)it.next()));
            }
        }
    }
    
    /**
     * Provides the <code>MenuDescription</code> according to the
     * given menu definition.
     * @param menu The menu definition
     * @return The corresponding Menu configuration.
     */
    protected MenuDescription getMenuDescription(Menu menu){
        MenuDescription menuDesc = null;
        if(menu!=null){
            menuDesc = new BasicMenuDescription();
            menuDesc.setInheritTreeMenu(menu.getInheritTreeMenu().equals("true"));
            menuDesc.setInheritTypeMenu(menu.getInheritTypeMenu().equals("true"));
            feedMenuDescription(menu.getItemList(), menuDesc);
        }
        return menuDesc;
    }
        
    // ==================================================================
    //
    // Public methods for NodeParser interface.
    //
    // ==================================================================    

    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.parser.api.NodeParser#parseNode(java.lang.Object, org.objectweb.util.explorer.explorerConfig.Node)
     */
    public void parseNode(Object key, Node node) {
        MenuDescription menuDesc = getMenuDescription(node.getMenu());
        if(menuDesc!=null){
            getFeeder().feed(ExplorerConstants.MENU_PROPERTY, key, menuDesc);
        }
    }

}


