/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.parser.lib;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipException;
import java.util.zip.ZipFile;

import org.objectweb.util.explorer.core.common.lib.BasicLoggable;
import org.objectweb.util.explorer.core.common.lib.ClassResolver;
import org.objectweb.util.explorer.explorerConfig.Zip;
import org.objectweb.util.explorer.explorerConfig.beans.ExplorerBean;
import org.objectweb.util.explorer.parser.api.ExplorerParser;

/**
 * This component manages the "zip" XML element.
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public class ZipManager 
     extends BasicLoggable 
  implements ExplorerParser 
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    /** The directory in which the zip file will be extracted. */    
    protected File extractedFolder_ = null;
    
    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * Default constructor
     */
    public ZipManager(){
        // In order to remove the temporary folder before stopping the program
        System.runFinalizersOnExit(true);
    }
    
    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Removes the given file and recursively removes if the target file is a directory.
     */
    protected void remove(File file){
        if(file!=null)  {
            if(file.isDirectory()){
                File[] children = file.listFiles();
                for(int i = 0 ; i < children.length ; i++) 
                    remove(children[i]);
            }
            file.delete();
        } 
    }
    
    /**
     * Writes a file on the given input stream.
     * Note that the stream are closed when leaving the this method.
     */
    protected void writeFile(InputStream input, OutputStream output) throws IOException {
        int bufferSize = 1024;
        byte[] buffer = new byte[bufferSize];
        int length=0;
        while((length = input.read(buffer))>0){
            output.write(buffer, 0, length);
        }
        output.close();
        input.close(); 
    }
    
    /**
     * Extracts a ZipEntry into the tmp dir.
     * @param file The zip file.
     * @param entry The entry to extract.
     * @throws IOException Throws when an IOException occured.
     */
    protected void extractEntry(ZipFile file, ZipEntry entry, String fileName) throws IOException {
        writeFile(file.getInputStream(entry), new FileOutputStream(fileName));
    }

    /*
     * It seems there is a problem with the methods "mkdirs()" and "deleteOnExit()" when they are working together.
     * Indeed, the mkdirs method recursively creates the directory contained in this File definition and the deleteOnExit method
     * removes those files when the virtual machine terminates, but when more than one directory is defined in the File, the first one
     * is not removed by the deleteOnExit calling.
     * It is the only problem that prevents this class to remove the "finalize()" method. 
     */
    /**
     * Creates the given directory.
     * @param path The path of the disrectory to create
     */
    protected void createDirectory(String path){
        File directory = new File(path);
        if(!directory.exists()){
            // the mkdirs method (used with deleteOnExit) doesn't remove the first directory of a directories path.
            directory.deleteOnExit();  
            directory.mkdirs();
        }
    }
    
    /**
     * Extracts a Zip archive
     */
    protected void extractFile(File file, Vector javaArchives) throws IOException {
        ZipFile zipFile = null;
        try {
            zipFile = new ZipFile(file.toString());
        } catch (ZipException e) {
            // Nothing to do
        } finally {
            file.deleteOnExit();
        }
        if(zipFile!=null){
            for(Enumeration entries = zipFile.entries();entries.hasMoreElements();){
                ZipEntry zipEntry = (ZipEntry)entries.nextElement();
                if(zipEntry.isDirectory()){
                    createDirectory(extractedFolder_.getCanonicalPath() + "/" + zipEntry.getName());
                } else {
                    // Creates the directories
                    int indice = zipEntry.getName().lastIndexOf('/');
                    if(indice!=-1){
                        createDirectory(extractedFolder_.getCanonicalPath() + "/" + zipEntry.getName().substring(0,indice));
                    }
                    // Computes the name of the file
                    String fileName = extractedFolder_.getCanonicalPath() + "/" + zipEntry.getName();
                    // Extracts the entry
                    try {
                        extractEntry(zipFile, zipEntry, fileName);
                    } catch (IOException e) {
                        getTrace().warn("IOException: " + e.getMessage());
                        e.printStackTrace();
                    }
                    if(zipEntry.getName().toLowerCase().endsWith(".jar")) {
                        // Adds the java archive into the vector containing the Java archive to load
                        File jarFile = new File(fileName);
                        jarFile.deleteOnExit();
                        try {
                            javaArchives.add(jarFile.toURL());
                        } catch (IOException e) {
                            getTrace().warn("IOException: " + e.getMessage());
                            e.printStackTrace();
                        }
                    } else {
                        extractFile(new File(fileName), javaArchives);
                    }
                }
            }
        }
    }
    
    /**
     * Loads the contained jar archives of the given zip element.
     * @param zip A zip archive containing the jar archives to load.
     */
    protected void loadZip(URL zipURL) throws IOException {
        if(zipURL!=null){
            Vector javaArchives = new Vector();

            // Copies the zip file into the java temp directory
            String fileName = extractedFolder_.getCanonicalPath() + "/temp.zip";
            writeFile(zipURL.openStream(), new FileOutputStream(fileName));

            // Extracts the zip file     
            File zipFile = new File(fileName);
            zipFile.deleteOnExit();                               
            extractFile(zipFile, javaArchives);
            
            // Loads the jar contained in the given zip file
            ClassResolver.addContext((URL[])javaArchives.toArray(new URL[0]));
        }
    }
    
    /**
     * Unzips all the zip archives of the given list and for each archive, 
     * loads the contained jar archives.
     * @param zipList A list of zip archives containing the jar archives to load.
     */
    protected void loadZips(List zipList){
        if(zipList!=null && zipList.size() > 0){
            if(extractedFolder_== null){
                // Ceates a temp directory
                String tmpDir = System.getProperty("java.io.tmpdir");
                if ((!tmpDir.endsWith("/")) && (!tmpDir.endsWith("\\")))
                    tmpDir = tmpDir + "/";
                extractedFolder_ = new File(tmpDir + "explorer" + System.currentTimeMillis());
                extractedFolder_.mkdir();
                extractedFolder_.deleteOnExit();        
            }            
            for(Iterator i = zipList.iterator() ; i.hasNext() ; ){
                Zip zip = (Zip)i.next();
                try {
                    loadZip(new URL(zip.getUrl()));
                } catch (MalformedURLException e) {
                    getTrace().warn("MalformedURLException: " + e.getMessage());
                } catch (IOException e) {
                    getTrace().warn("IOException: " + e.getMessage());
                }
            }
        }
    }    
    
    // ==================================================================
    //
    // Public methods for ExplorerParser interface. 
    //
    // ==================================================================    
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.parser.api.ExplorerParser#parseExplorer(org.objectweb.util.explorer.explorerConfig.beans.ExplorerBean)
     */
    public void parseExplorer(ExplorerBean explorer) {
        loadZips(explorer.getZipList());
    }

    // ==================================================================
    //
    // Public method surcharging java.lang.Object methods.
    //
    // ==================================================================
    
    /**
     * Removes the temporary folder containing the extracting files.
     * Overriding methods
     * @see java.lang.Object#finalize()
     */
    public void finalize() throws Throwable {
        if(extractedFolder_!=null && extractedFolder_.exists()){
            remove(extractedFolder_);
        }
    }    
}
