/*====================================================================

Objectweb Explorer Framework
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Moroy, Philippe Merle.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.util.explorer.resolver.lib;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.util.explorer.api.Entry;
import org.objectweb.util.explorer.core.common.api.BadParamException;
import org.objectweb.util.explorer.core.common.api.Description;
import org.objectweb.util.explorer.core.common.api.KeyProvider;
import org.objectweb.util.explorer.core.common.lib.BindingFeature;
import org.objectweb.util.explorer.core.common.lib.ClassesInheritance;
import org.objectweb.util.explorer.core.role.api.RoleManager;
import org.objectweb.util.explorer.core.role.api.RoleProvider;
import org.objectweb.util.explorer.property.api.PropertyProvider;
import org.objectweb.util.explorer.resolver.api.PropertyResolver;

/**
 * 
 *
 * @author <a href="mailto:Jerome.Moroy@lifl.fr">Jrme Moroy</a>,
 *         <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>.  
 * 
 * @version 0.1
 */
public abstract class AbstractPropertyResolver 
              extends BindingFeature
           implements PropertyResolver              
{

    //==================================================================
    //
    // Internal States.
    //
    // ==================================================================

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal method.
    //
    // ==================================================================
    
    /**
     * Provides the Id of the typeSystem.
     * @return The id of the type system.
     */
    protected String getTypeSystemId(){
        return "Java";
    }
    
    /**
     * Provides the KeyProvider.
     */
    protected KeyProvider getKeyProvider(){
        try {
            return (KeyProvider)lookupFc(KeyProvider.KEY_PROVIDER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(KeyProvider.KEY_PROVIDER + ": interface not found!");
            return null;
        }
    }
    
    /**
     * Provides the PropertyProvider.
     */
    protected PropertyProvider getPropertyProvider(){
        try {
            return (PropertyProvider)lookupFc(PropertyProvider.PROPERTY_PROVIDER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(PropertyProvider.PROPERTY_PROVIDER + ": interface not found!");
            return null;
        }
    }    

    /**
     * Provides the RoleManager.
     */
    protected RoleManager getRoleManager(){
        try {
            return (RoleManager)lookupFc(RoleManager.ROLE_MANAGER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(RoleManager.ROLE_MANAGER + ": interface not found!");
            return null;
        }
        
    }
    
    /**
     * Provides the RoleProvider.
     */
    protected RoleProvider getRoleProvider(){
        try {
            return (RoleProvider)lookupFc(RoleProvider.ROLE_PROVIDER);
        } catch (NoSuchInterfaceException e) {
            getTrace().warn(RoleProvider.ROLE_PROVIDER + ": interface not found!");
            return null;
        }
        
    }    
    
    /**
     * Tries to provide a description for the given entry.
     * The corresponfing algorithm is the following one:
     * <pre>
     *   c &lt;- getInheritedClasses();
     *   r &lt;- getInheritedRoles();
     *   for i in c
     *     for j in r
     *       begin
     *         key &lt;- computeKey(c[i],r[j])
     *         getPropertyDescriptionDescription(key);
     *       end
     *     end for j
     *   end for i 
     * </pre>
     * @param entry The entry for which a description needs to be found
     * @see org.objectweb.util.explorer.core.common.lib.ClassesInheritance#getInheritedClasses(Class)
     * @see org.objectweb.util.explorer.core.role.api.RoleProvider#getInheritedRoleIds(String[])
     * @see org.objectweb.util.explorer.core.common.api.KeyProvider#computeKey(Object)
     * @see org.objectweb.util.explorer.property.api.PropertyProvider#getPropertyDescription(String, Object)
     * @return The associated description (or null if no description is found).
     */
    protected Description getDescription(Entry entry){
        if(entry!=null && entry.getValue()!=null){
            Class[] javaTypes = ClassesInheritance.getInheritedClasses(entry.getValue().getClass());
            String[] roleIds = getRoleProvider().getInheritedRoleIds(getRoleManager().getCurrentRoleIds());
            for (int i = 0; i < javaTypes.length; i++) {
                for (int j = 0; j < roleIds.length; j++) {
                    Object key = null;
                    try {
                        key = getKeyProvider().computeKey(new String[]{getTypeSystemId(),javaTypes[i].getName(),roleIds[j]});
                    } catch (BadParamException e) {
                        getTrace().warn("Bad key param: " + e.getMessage());
                    }
                    if(key!=null){
                        Description desc = (Description)getPropertyProvider().getPropertyDescription(getPropertyType(),key);
                        if(desc!=null){
                            return desc;
                        }
                    }
                }
            }
        }
        return null;
    }    

    /**
     * Provides the name of the property to resolve.
     * @return The name of the property
     * @see org.objectweb.util.explorer.core.common.api.ExplorerConstants
     */
    protected abstract String getPropertyType();
    
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.resolver.api.PropertyResolver#resolve(java.lang.String, org.objectweb.util.explorer.api.Entry)
     */
    public Description resolve(String propertyType, Entry currentEntry) {
        return resolve(getPropertyType(), currentEntry, null);
    }    
    
    // ==================================================================
    //
    // Public methods for BindingFeature abstract class. 
    //
    // ==================================================================    
        
    /* (non-Javadoc)
     * @see org.objectweb.util.explorer.core.common.lib.BindingFeature#clientFc()
     */
    public String[] clientFc() {
        return new String[]{PropertyProvider.PROPERTY_PROVIDER, 
                KeyProvider.KEY_PROVIDER, 
                RoleManager.ROLE_MANAGER, 
                RoleProvider.ROLE_PROVIDER};
    }

}


