/*====================================================================

 Objectweb Explorer Framework
 Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
 Contact: openccm@objectweb.org

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA

 Initial developer(s): Latitia Lafeuille.
 Contributor(s): ______________________________________.

 ====================================================================*/
package org.objectweb.util.explorer.swing.graph;

import java.awt.Dimension;
import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphCell;
import org.jgraph.graph.Edge;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphModel;

/**
 * 
 * @version 0.2
 */
public class CompositeVertex 
     extends PrimitiveVertex 
{

    /** If the vertex is open, the elements inside are visible */
    private boolean isOpen = false;

    /** List of the verteces inside the composite */
    private ArrayList verteces;

    /** The origin point of the vertex */
    private Point origin;

    /** List of the edges inside the composite */
    private ArrayList binding;

    /** The list of the internal ports */
    private ArrayList internalPorts;

    private ArrayList internalServerPorts;

    private ArrayList internalClientPorts;

    private ArrayList internalControllerPorts;

    /** Parameters that serve for attribute a number for each internal ports */
    private int numberInternalServer;

    private int numberInternalClient;

    private int numberInternalController;

    /**
     * Create a new Component Vertex
     * 
     * @param component the component associated with the vertex
     * @param name the name of the component
     * @param graph the graph that contain the vertex
     * @param compositeSize the size of the composite
     * @param isOpen a boolean that indicate if the component is open
     * @param controllersDisplay a boolean that indicate if the controller ports are visible
     * @param isStarted a boolean that indicate if the component is started
     * @param isShared a boolean that indicate if the component is shared
     */
    public CompositeVertex(Object component, String name, Graph graph,
            Dimension compositeSize, boolean isOpen,
            boolean controllersDisplay, boolean isStarted, boolean isShared) {
        super(component, name, graph, compositeSize, controllersDisplay,
                isStarted, isShared);
        verteces = new ArrayList();
        binding = new ArrayList();
        setSize(compositeSize);
        this.isOpen = isOpen;
        this.type = VertexType.COMPOSITE_VERTEX;
        if (isOpen) {
            verteces.add(this);
            AttributeMap vAttributes = graph.getModel().createAttributes();
            internalPorts = new ArrayList();
            internalServerPorts = new ArrayList();
            internalClientPorts = new ArrayList();
            internalControllerPorts = new ArrayList();
            origin = new Point();
        }
    }

    /**
     * 
     * @return the vertex edges list
     */
    public ArrayList getBinding() {
        return binding;
    }

    /**
     * 
     * @param model
     * @return An edge that have been created and inserted into the edges list
     *         of the vertex
     */
    public Edge addBinding(GraphModel model) {
        Edge e = new DefaultEdge();
        binding.add(e);
        AttributeMap eAttributes = model.createAttributes();
        attributes.put(e, eAttributes);
        int style = GraphConstants.STYLE_ORTHOGONAL;
        GraphConstants.setLineStyle(eAttributes, style);
        return e;
    }

    /**
     * Research an internal port of a composite vertex with his name
     * 
     * @param name the name of the researched port
     * @return the researched port
     */
    public MyPort getInternalPort(String name) {
        for (int i = 0; i < internalPorts.size(); i++) {
            if ((((MyPort) internalPorts.get(i)).getName().equals(name))
                    && (!((MyPort) internalPorts.get(i)).isGenerated()))
                return (MyPort) internalPorts.get(i);
        }
        return null;
    }

    /**
     * Research a port generated (a port is generated automaticly when an
     * internal port is created) with his name
     * 
     * @param name the name of the researched port
     * @return the researched port
     */
    public MyPort getGeneratedPort(String name) {
        ArrayList internalPorts = getListInternalPorts();
        for (int i = 0; i < internalPorts.size(); i++) {
            if ((((MyPort) internalPorts.get(i)).getName().equals(name))
                    && (((MyPort) internalPorts.get(i)).isGenerated()))
                return (MyPort) internalPorts.get(i);
        }
        return null;
    }

    /**
     * 
     * @return the vertex internal ports list
     */
    public ArrayList getListInternalPorts() {
        return internalPorts;
    }

    /**
     * 
     * @return the vertex internal server ports list
     */
    public ArrayList getListInternalServerPorts() {
        if (internalServerPorts.size() == 0) {
            List ports = getListInternalPorts();
            for (int i = 0; i < ports.size(); i++) {
                if (((MyPort) ports.get(i)).getType().equals(
                        PortType.SERVER_PORT))
                    internalServerPorts.add(ports.get(i));
            }
        }
        return internalServerPorts;
    }

    /**
     * 
     * @return the vertex internal client ports list
     */
    public ArrayList getListInternalClientPorts() {
        if (internalClientPorts.size() == 0) {
            List ports = getListInternalPorts();
            for (int i = 0; i < ports.size(); i++) {
                if (((MyPort) ports.get(i)).getType().equals(
                        PortType.CLIENT_PORT))
                    internalClientPorts.add(ports.get(i));
            }
        }
        return internalClientPorts;
    }

    /**
     * 
     * @return the vertex internal controller ports list
     */
    public ArrayList getListInternalControllerPorts() {
        if (internalControllerPorts.size() == 0) {
            List ports = getListInternalPorts();
            for (int i = 0; i < ports.size(); i++) {
                if (((MyPort) ports.get(i)).getType().equals(
                        PortType.CONTROLLER_PORT))
                    internalControllerPorts.add(ports.get(i));
            }
        }
        return internalControllerPorts;
    }

    /**
     * Research a vertex inside the composite with his name
     * 
     * @param name the name of the researched vertex
     * @return the vertex researched
     */
    public PrimitiveVertex getVertex(String name) {
        for (int i = 0; i < verteces.size(); i++) {
            if (((PrimitiveVertex) verteces.get(i)).getName().equals(name))
                return (PrimitiveVertex) verteces.get(i);
        }
        return null;
    }

    /**
     * Add a vertex inside the composite
     * 
     * @param component the component associated with the vertex inserted
     * @param name the name of the vertex inserted
     * @param graph the graph that contain the verteces
     * @param type the type of the vertex inserted
     * @param isOpen
     * @param controllersDisplay
     * @param isStarted
     * @param isShared
     */
    public void addVertex(Object component, String name, Graph graph,
            String type, boolean isOpen, boolean controllersDisplay,
            boolean isStarted, boolean isShared) {
        PortGraphicsInterface pg = Graph.getPortGraphics();
        VertexGraphicsInterface vg = Graph.getVertexGraphics();
        DefaultGraphCell vertex;
        if (type.equals(VertexType.PRIMITIF_VERTEX)) {
            vertex = new PrimitiveVertex(component, name, graph, getSize(),
                    controllersDisplay, isStarted, isShared);
            AttributeMap vAttributes = graph.getModel().createAttributes();
            attributes.put(vertex, vAttributes);
        } else {
            vertex = new CompositeVertex(component, name, graph, vg
                    .getDefaultSize(), isOpen, controllersDisplay, isStarted,
                    isShared);
        }
        verteces.add(vertex);
    }

    /**
     * Add a port on a vertex inside the composite vertex
     * 
     * @param vertexName the vertex inside the composite
     * @param itf the interface associated with the port inserted
     * @param portName the name of the port inserted
     * @param type the type of the port inserted
     */
    public void addSubComponentPort(String vertexName, Object itf,
            String portName, String type) {
        PrimitiveVertex v = getVertex(vertexName);
        v.addPort(itf, portName, type);
    }

    /**
     * Add a port on the composite vertex
     * 
     * @param itf the interface associated with the port inserted
     * @param portName the name of the port inserted
     * @param type the type of the port inserted
     */
    public void addCompositePort(Object itf, String portName, String type) {
        MyPort initialPort = null;
        MyPort generatedPort = null;
        if ((type.equals(PortType.CLIENT_PORT))
                || (type.equals(PortType.COLLECTION_PORT))) {
            numberInternalClient++;
            initialPort = new MyPort(itf, portName, type, numberInternalClient,
                    false);
            generatedPort = new MyPort(itf, portName, PortType.SERVER_PORT,
                    numberInternalClient, true);
            internalPorts.add(generatedPort);
            super.add(generatedPort);
        } else if (type.equals(PortType.SERVER_PORT)) {
            numberInternalServer++;
            initialPort = new MyPort(itf, portName, type, numberInternalServer,
                    false);
            generatedPort = new MyPort(itf, portName, PortType.CLIENT_PORT,
                    numberInternalServer, true);
            internalPorts.add(generatedPort);
            super.add(generatedPort);
        } else if (type.equals(PortType.CONTROLLER_PORT)) {
            numberInternalController++;
            initialPort = new MyPort(itf, portName, type,
                    numberInternalController, false);
        }
        internalPorts.add(initialPort);
        super.add(initialPort);
    }

    /**
     * @return <tt>true</tt> if the elements inside are visible.
     */
    public boolean isOpen() {
        return isOpen;
    }

    /**
     * @param isOpen <tt>true</tt> if the elements inside are visible.
     */
    public void setOpen(boolean isOpen) {
        this.isOpen = isOpen;
    }

    /**
     * 
     * @return the verteces list inside the composite vertex
     */
    public ArrayList getVerteces() {
        return verteces;
    }

    /**
     * 
     * @return the graphics origin of the vertex
     */
    public Point getOrigin() {
        return origin;
    }

    /**
     * @param p The graphics origin of the vertex
     */
    public void setOrigin(Point p) {
        origin.x = p.x;
        origin.y = p.y;
    }

}