/*====================================================================

 Objectweb Explorer Framework
 Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
 Contact: openccm@objectweb.org

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA

 Initial developer(s): Latitia Lafeuille.
 Contributor(s): ______________________________________.

 ====================================================================*/
package org.objectweb.util.explorer.swing.graph;

import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Map;
import java.util.Vector;

import org.jgraph.JGraph;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.CellMapper;
import org.jgraph.graph.CellView;
import org.jgraph.graph.ConnectionSet;
import org.jgraph.graph.DefaultEdge;
import org.jgraph.graph.DefaultGraphModel;
import org.jgraph.graph.Edge;
import org.jgraph.graph.GraphConstants;
import org.jgraph.graph.GraphModel;
import org.jgraph.graph.PortView;
import org.jgraph.graph.VertexView;

/**
 * 
 * @version 0.2
 */
public class Graph extends JGraph {

    /** The connectionSet that serves to connect edges */
    private ConnectionSet cs = new ConnectionSet();

    private GraphModel model = new DefaultGraphModel();

    /** The interface where are defined the graphics method to draw a port */
    private static PortGraphicsInterface pg;

    /** The interface where are defined the graphics method to draw a vertex */
    private static VertexGraphicsInterface vg;

    /** The Nested Map (from Cells to Attributes) */
    private Map attributes;

    /** The super component in the graph (the open composite vertex) */
    private CompositeVertex superComponent;

    /** The list of the verteces where they aren't into an open composite vertex */
    private ArrayList primitiveVerteces;

    /**
     * The edges list contained in a graph where there hasn't an open composite
     * vertex
     */
    private ArrayList primitiveEdges;

    /**
     * The vector that define the position of the verteces. It's used in the
     * layout algorithm
     */
    private Vector vertecesPosition = new Vector();

    /**
     * Default constructor
     * @param model The graph model 
     */
    public Graph(GraphModel model) {
        super(model);
        attributes = new Hashtable();
        this.model = model;
        primitiveVerteces = new ArrayList();
        primitiveEdges = new ArrayList();
    }

    /**
     * Default constructor
     * @param model The model
     * @param portGraphics The graphic port
     * @param vertexGraphics The graphic vertex
     */
    public Graph(GraphModel model, PortGraphicsInterface portGraphics,
            VertexGraphicsInterface vertexGraphics) {
        super(model);
        this.model = model;
        attributes = new Hashtable();
        cs = new ConnectionSet();
        setPortGraphics(portGraphics);
        setVertexGraphics(vertexGraphics);
        primitiveVerteces = new ArrayList();
        primitiveEdges = new ArrayList();
    }

    /**
     * Method that inserts all the objects created (verteces, ports, edges, ..)
     * into the model. We have to call this method to display the graph.
     * 
     * @param isPrimitif boolean that indicate if there is an open composite vertex in
     *            the graph
     */
    public void insertIntoModel(boolean isPrimitif) {
        Object[] cells;
        if (!isPrimitif) {
            ArrayList edges = superComponent.getBinding();
            ArrayList verteces = superComponent.getVerteces();
            cells = new Object[verteces.size() + edges.size()];

            int j = 0;
            for (j = 0; j < verteces.size(); j++)
                cells[j] = verteces.get(j);

            for (int k = j; k < j + edges.size(); k++)
                cells[k] = edges.get(k - j);
        } else {
            ArrayList edges = primitiveEdges;
            ArrayList verteces = primitiveVerteces;
            cells = new Object[verteces.size() + edges.size()];

            int j = 0;
            for (j = 0; j < verteces.size(); j++)
                cells[j] = verteces.get(j);

            for (int k = j; k < j + edges.size(); k++)
                cells[k] = edges.get(k - j);
        }
        model.insert(cells, attributes, cs, null, null);
    }

    /**
     * The method updates the vector vertecesPosition (when it's empty) that
     * serves for the layoutALgorithm. The method is called when an edge is
     * connected.
     * 
     * @param sourcePort the source port of the connected edge
     * @param targetPort the target port of the connected edge
     * @param isCompositeEdge boolean that indicates if the edges is connected to an open
     *            composite vertex
     */
    private void updateEmptyVertecesPosition(MyPort sourcePort,
            MyPort targetPort, boolean isCompositeEdge) {
        Object source = sourcePort.getParent();
        Object target = targetPort.getParent();
        if (isCompositeEdge) {
            if ((source instanceof CompositeVertex)
                    && (((CompositeVertex) source).isOpen())) {
                if (!((target instanceof CompositeVertex) && (((CompositeVertex) target)
                        .isOpen()))) {
                    vertecesPosition.add(0, new Vector());
                    Vector newLine = (Vector) vertecesPosition.get(0);
                    newLine.setSize(1);
                    newLine.add(0, target);
                }
            } else if (((target instanceof CompositeVertex) && (((CompositeVertex) target)
                    .isOpen()))
                    && (!((source instanceof CompositeVertex) && (((CompositeVertex) source)
                            .isOpen())))) {
                vertecesPosition.add(0, new Vector());
                Vector newLine = (Vector) vertecesPosition.get(0);
                newLine.setSize(1);
                newLine.add(0, source);
            }
        } else if (!(sourcePort.getType().equals(PortType.CONTROLLER_PORT))) {
            //add a line
            vertecesPosition.add(0, new Vector());
            Vector newLine = (Vector) vertecesPosition.get(0);
            newLine.setSize(2);
            //add the two verteces in the line
            newLine.add(0, source);
            newLine.add(1, target);
        } else {
            //add 2 lines vertecesPosition.add(0, new Vector());
            vertecesPosition.add(0, new Vector());
            Vector newLine1 = (Vector) vertecesPosition.get(0);
            newLine1.setSize(1);
            vertecesPosition.add(1, new Vector());
            Vector newLine2 = (Vector) vertecesPosition.get(1);
            newLine2.setSize(2);
            //add the two verteces in the lines
            newLine1.add(0, source);
            newLine2.add(1, target);
        }
    }

    /**
     * The method updates the vector vertecesPosition when the method is called
     * in the moment that a composite edge is connecting. The method is called
     * when an edge is connected.
     * 
     * @param sourcePort the source port of the connected edge
     * @param targetPort the target port of the connected edge
     */
    private void updateVertecesPositionWithCompositeEdge(MyPort sourcePort,
            MyPort targetPort) {
        Object source = sourcePort.getParent();
        Object target = targetPort.getParent();
        if ((source instanceof CompositeVertex)
                && (((CompositeVertex) source).isOpen())) {
            if (!((target instanceof CompositeVertex) && (((CompositeVertex) target)
                    .isOpen()))) {
                int index = sourcePort.getNumber();
                if (!(haveTarget(target))) {
                    if (vertecesPosition.size() > index) {
                        Vector targetLine = (Vector) vertecesPosition
                                .get(index);
                        if (targetLine.get(0) == null)
                            targetLine.setElementAt(target, 0);
                        else
                            targetLine.add(0, target);
                    } else {
                        int oldSize = vertecesPosition.size();
                        int newSize = index + 1;
                        vertecesPosition.setSize(index + 1);
                        for (int i = oldSize; i < vertecesPosition.size(); i++) {
                            Vector newLine = new Vector();
                            newLine.setSize(1);
                            vertecesPosition.setElementAt(newLine, i);
                        }
                        ((Vector) vertecesPosition.get(index)).setElementAt(
                                target, 0);
                    }
                }
            }
        } else if ((target instanceof CompositeVertex)
                && (((CompositeVertex) target).isOpen())) {
            if (!((target instanceof CompositeVertex) && (((CompositeVertex) target)
                    .isOpen()))) {
                int index = targetPort.getNumber();
                if (vertecesPosition.size() > index) {
                    Vector sourceLine = (Vector) vertecesPosition.get(index);
                    if (sourceLine.get(sourceLine.size()) == null)
                        sourceLine.setElementAt(source, sourceLine.size());
                    else {
                        int oldSize = sourceLine.size();
                        sourceLine.setSize(oldSize + 1);
                        sourceLine.add(oldSize, source);
                    }
                } else {
                    int oldSize = vertecesPosition.size();
                    vertecesPosition.setSize(index + 1);
                    for (int i = 0; i < vertecesPosition.size(); i++) {
                        Vector newLine = new Vector();
                        newLine.setSize(1);
                        vertecesPosition.add(i, newLine);
                    }
                    Vector sourceLine = (Vector) vertecesPosition.get(index);
                    sourceLine.setSize(getNbColumn() + 1);
                    sourceLine.add(getNbColumn(), source);
                }
            }
        }

    }

    /**
     * @return the number of columns in the verteces position vector
     */
    private int getNbColumn() {
        int nb = 0;
        for (int i = 0; i < vertecesPosition.size(); i++) {
            Vector line = (Vector) vertecesPosition.get(i);
            int lineSize = 0;
            for (int j = 0; j < line.size(); j++) {
                if (line.get(j) != null)
                    lineSize++;
            }
            if (lineSize > nb) {
                nb = lineSize;
            }
        }
        return nb;
    }

    /**
     * @param source the source researched in the veteces position vector
     * @return boolean that indicates if the source is inside the vector
     */
    private boolean haveSource(Object source) {
        int haveSource = 0;
        for (int i = 0; i < vertecesPosition.size(); i++) {
            if (((Vector) vertecesPosition.get(i)).contains(source))
                haveSource++;
        }
        return (haveSource != 0);
    }

    /**
     * @param target the target researched in the veteces position vector
     * @return boolean that indicates if the taget is inside the vector
     */
    private boolean haveTarget(Object target) {
        int haveTarget = 0;
        for (int i = 0; i < vertecesPosition.size(); i++) {
            if (((Vector) vertecesPosition.get(i)).contains(target))
                haveTarget++;
        }
        return (haveTarget != 0);
    }

    /**
     * Add a target after a source int the verteces position vector
     * 
     * @param vertecesPosition the vector
     * @param sourceIndexLine
     * @param sourceIndex
     * @param target the target to insert
     */
    private void addTargetAfter(Vector vertecesPosition, int sourceIndexLine,
            int sourceIndex, Object target) {
        int add = 0;
        for (int i = sourceIndexLine; i < vertecesPosition.size(); i++) {
            Vector line = (Vector) vertecesPosition.get(i);
            if (line.size() < sourceIndex + 2) {
                line.setSize(sourceIndex + 2);
                line.add(sourceIndex + 1, target);
                add++;
            } else if (line.get(sourceIndex + 1) == null) {
                line.setElementAt(target, sourceIndex + 1);
                add++;
            }
        }
        if (add == 0) {
            vertecesPosition.setSize(vertecesPosition.size() + 1);
            vertecesPosition.setElementAt(new Vector(),
                    vertecesPosition.size() - 1);
            Vector targetLine = (Vector) vertecesPosition.get(vertecesPosition
                    .size() - 1);
            targetLine.setSize(sourceIndex + 2);
            targetLine.setElementAt(target, sourceIndex + 1);
        }
    }

    /**
     * Add a source before a target int the verteces position vector
     * 
     * @param vertecesPosition the vector
     * @param targetIndexLine
     * @param targetIndex
     * @param source the source to insert
     */
    private void addSourceBefore(Vector vertecesPosition, int targetIndexLine,
            int targetIndex, Object source) {
        int add = 0;
        for (int i = targetIndexLine; i < vertecesPosition.size(); i++) {
            Vector line = (Vector) vertecesPosition.get(i);
            if (line.size() < targetIndex) {
                line.setSize(targetIndex);
                line.add(line.size() - 1, source);
                add++;
            } else if (line.get(targetIndex - 1) == null) {
                ((Vector) vertecesPosition.get(i)).setElementAt(source,
                        targetIndex - 1);
                add++;
            }
        }
        if (add == 0) {
            vertecesPosition.setSize(vertecesPosition.size() + 1);
            vertecesPosition.setElementAt(new Vector(),
                    vertecesPosition.size() - 1);
            Vector sourceLine = (Vector) vertecesPosition.get(vertecesPosition
                    .size() - 1);
            sourceLine.setSize(targetIndex);
            sourceLine.add(targetIndex - 1, source);
        }
    }

    /**
     * @param vertecesPosition
     * @param vertex
     * @return the index of the line that contains the vertex
     */
    private int getIndexLineOf(Vector vertecesPosition, Object vertex) {
        int index = 0;
        for (int i = 0; i < vertecesPosition.size(); i++) {
            if (((Vector) vertecesPosition.get(i)).contains(vertex))
                index = i;
        }
        return index;
    }

    /**
     * Updates the vertecesPosition vector that serve for the layout algorithm
     * 
     * @param edge
     * @param sourcePort
     * @param targetPort
     * @param isCompositeEdge
     */
    public void updateVertecesPosition(Object edge, MyPort sourcePort,
            MyPort targetPort, boolean isCompositeEdge) {
        if ((sourcePort != null) && (targetPort != null)) {
            Object source = sourcePort.getParent();
            Object target = targetPort.getParent();

            if (vertecesPosition.isEmpty()) {
                updateEmptyVertecesPosition(sourcePort, targetPort,
                        isCompositeEdge);
            } else {
                if (isCompositeEdge) {
                    updateVertecesPositionWithCompositeEdge(sourcePort,
                            targetPort);
                } else {
                    if (!(sourcePort.getType().equals(PortType.CONTROLLER_PORT))) {
                        //if the source and the target not in the
                        // vertecesPosition
                        //add a new line in the vertecesPosition
                        //and add the two verteces in this line
                        if ((!haveSource(source)) && (!haveTarget(target))) {
                            //add a line
                            int newIndexLine = vertecesPosition.size();
                            vertecesPosition.add(newIndexLine, new Vector());
                            Vector newLine = (Vector) vertecesPosition
                                    .get(newIndexLine);
                            newLine.setSize(2);
                            //add the two verteces in the line
                            newLine.add(0, source);
                            newLine.add(1, target);
                        } else if (!haveSource(source)) {
                            int targetIndexLine = getIndexLineOf(
                                    vertecesPosition, target);
                            Vector targetLine = (Vector) vertecesPosition
                                    .get(targetIndexLine);
                            int targetIndex = targetLine.indexOf(target);
                            if (targetIndex == 0) {
                                targetLine.add(0, source);
                            } else if (targetLine.get(targetIndex - 1) == null) {
                                targetLine
                                        .setElementAt(source, targetIndex - 1);
                            } else {
                                addSourceBefore(vertecesPosition,
                                        targetIndexLine, targetIndex, source);
                            }
                        } else if (!(haveTarget(target))) {
                            int sourceIndexLine = getIndexLineOf(
                                    vertecesPosition, source);
                            Vector sourceLine = (Vector) vertecesPosition
                                    .get(sourceIndexLine);
                            int sourceIndex = sourceLine.indexOf(source);
                            if (sourceIndex == sourceLine.size()) {
                                sourceLine.setSize(sourceIndex + 2);
                                sourceLine.add(sourceLine.size() - 1, target);
                            } else if ((sourceLine.size() > sourceIndex + 1)
                                    && (sourceLine.get(sourceIndex + 1) == null)) {
                                sourceLine.add(sourceIndex + 1, target);
                            } else {
                                addTargetAfter(vertecesPosition,
                                        sourceIndexLine, sourceIndex, target);
                            }
                        }
                        //else, the target and the source are already in the
                        // vertecesPosition

                    } else {
                        if ((!(haveTarget(source))) && (!(haveTarget(target)))) {
                            //add 2 lines
                            int newIndexLine1 = vertecesPosition.size();
                            vertecesPosition.add(newIndexLine1, new Vector());
                            Vector newLine1 = (Vector) vertecesPosition
                                    .get(newIndexLine1);
                            int newIndexLine2 = vertecesPosition.size();
                            vertecesPosition.add(newIndexLine2, new Vector());
                            Vector newLine2 = (Vector) vertecesPosition
                                    .get(newIndexLine2);
                            newLine1.setSize(1);
                            newLine2.setSize(2);
                            //add the two verteces in the lines
                            newLine1.add(0, source);
                            newLine2.add(1, target);
                        } else if (!(haveTarget(source))) {
                            int targetIndexLine = getIndexLineOf(
                                    vertecesPosition, target);
                            Vector targetLine = (Vector) vertecesPosition
                                    .get(targetIndexLine);
                            int targetIndex = targetLine.indexOf(target);
                            if (targetIndexLine == 0) {
                                if (targetIndex > 0) {
                                    Vector sourceLine = new Vector();
                                    sourceLine.setSize(targetIndex);
                                    sourceLine.setElementAt(source,
                                            targetIndex - 1);
                                    vertecesPosition.add(0, sourceLine);
                                } else {
                                    Vector sourceLine = new Vector();
                                    sourceLine.setSize(1);
                                    sourceLine.setElementAt(source, 0);
                                    ((Vector) vertecesPosition
                                            .get(targetIndexLine)).add(0, null);
                                    vertecesPosition.add(0, sourceLine);
                                }
                            } else {
                                Vector sourceLine = ((Vector) vertecesPosition
                                        .get(targetIndexLine - 1));
                                if (sourceLine.size() < targetIndex) {
                                    sourceLine.setSize(targetIndex);
                                    sourceLine.setElementAt(source,
                                            targetIndex - 1);
                                } else {
                                    if (sourceLine.get(targetIndex - 1) == null)
                                        sourceLine.setElementAt(source,
                                                targetIndex - 1);
                                    else
                                        sourceLine.add(targetIndex - 1, source);
                                }
                            }
                        } else if (!(haveTarget(target))) {
                            int sourceIndexLine = getIndexLineOf(
                                    vertecesPosition, source);
                            Vector sourceLine = (Vector) vertecesPosition
                                    .get(sourceIndexLine);
                            int sourceIndex = sourceLine.indexOf(source);
                            addTargetAfter(vertecesPosition, sourceIndexLine,
                                    sourceIndex, target);
                        }
                    }
                }
            }
        }
    }

    /**
     * Apply the layout algorithm. This method is called when there is an open
     * composite vertex in the graph.
     * 
     * @param component the open composite vertex
     */
    public void applyLayout(CompositeVertex component) {
        LayoutAlgorithm algo = new LayoutAlgorithm(getVertecesPosition(),vg);
        if ((component.getOrigin().x == 0) && (component.getOrigin().y == 0))
            component.setOrigin(VertexGraphicsInterface.SUPER_COMPOSITE_ORIGIN);
        algo.applyLayout(this, component.getOrigin(), component.getSize(),
                component);
    }

    /**
     * Apply the layout algorithm. This method is called when there isn't an
     * open composite vertex in the graph.
     * 
     * @param frameSize the size of the frame
     */
    public void applyLayout(Dimension frameSize) {
        if (getVertecesPosition().size() != 0) {
            LayoutAlgorithm algo = new LayoutAlgorithm(getVertecesPosition(),vg);
            algo.applyLayout(this, VertexGraphicsInterface.PRIMITIVE_ORIGIN,
                    frameSize, null);
        }
    }

    /**
     * Add an open composite vertex int he graph
     * 
     * @param component the open composite vertex
     */
    public void addSuperComponent(CompositeVertex component) {
        AttributeMap componentAttributes = getModel().createAttributes();
        attributes.put(component, componentAttributes);
        superComponent = component;
    }

    /**
     * Add a primitive vertex or a closed composite vertex in the graph.
     * 
     * @param component the vertex
     */
    public void addPrimitiveComponent(PrimitiveVertex component) {
        AttributeMap vertexAttributes = model.createAttributes();
        attributes.put(component, vertexAttributes);
        Rectangle2D c1Bounds = vertexAttributes.createRect(10, 10, vg
                .getDefaultSize().width, vg.getDefaultSize().height);
        GraphConstants.setBounds(vertexAttributes, c1Bounds);
        primitiveVerteces.add(component);
    }

    /**
     * Add an edge in the graph in the case that the graph don't have an open
     * composite vertex.
     */
    public void addPrimitiveEdge() {
        DefaultEdge edge = new DefaultEdge();
        AttributeMap edgeAttrib = model.createAttributes();
        attributes.put(edge, edgeAttrib);
        int style = GraphConstants.STYLE_ORTHOGONAL;
        GraphConstants.setLineStyle(edgeAttrib, style);
    }

    /**
     * @return The attributes map.
     */
    public Map getAttributes() {
        return attributes;
    }

    /**
     * @return The position of the verteces
     */
    public Vector getVertecesPosition() {
        return vertecesPosition;
    }

    /**
     * @see org.jgraph.JGraph#getModel()
     */
    public GraphModel getModel() {
        return model;
    }

    /**
     * @return The graphic port.
     */
    public static PortGraphicsInterface getPortGraphics() {
        return pg;
    }

    /**
     * @return The graphic vertex.
     */
    public static VertexGraphicsInterface getVertexGraphics() {
        return vg;
    }

    /**
     * Connect an edge in the case that the graph have an open composite vertex
     * 
     * @param sourcePort
     * @param targetPort
     * @param isCompositeEdge
     * @param component
     */
    public void compositeConnect(MyPort sourcePort, MyPort targetPort,
            boolean isCompositeEdge, CompositeVertex component) {
        Edge edge = component.addBinding(model);
        cs.connect(edge, sourcePort, targetPort);
        updateVertecesPosition(edge, sourcePort, targetPort, isCompositeEdge);
    }

    /**
     * Connect an edge in the case that the graph don't have an open composite
     * vertex
     * 
     * @param sourcePort
     * @param targetPort
     * @param isCompositeEdge
     */
    public void primitiveConnect(MyPort sourcePort, MyPort targetPort,
            boolean isCompositeEdge) {
        Edge edge = new DefaultEdge();
        primitiveEdges.add(edge);
        AttributeMap eAttributes = model.createAttributes();
        attributes.put(edge, eAttributes);
        int style = GraphConstants.STYLE_ORTHOGONAL;
        GraphConstants.setLineStyle(eAttributes, style);
        GraphConstants.setDisconnectable(eAttributes, false);
        cs.connect(edge, sourcePort, targetPort);
        updateVertecesPosition(edge, sourcePort, targetPort, isCompositeEdge);
    }

    /**
     * @return The set of connections
     */
    public ConnectionSet getConnectionSet() {
        return cs;
    }

    /**
     * Fixes the graphic port
     * @param portGraphics The graphic port 
     */
    public void setPortGraphics(PortGraphicsInterface portGraphics) {
        pg = portGraphics;
    }

    /**
     * Fixes the graphic vertex
     * @param vertexGraphics The graphic vertex
     */
    public void setVertexGraphics(VertexGraphicsInterface vertexGraphics) {
        vg = vertexGraphics;
    }

    /**
     * Constructs a PortView view for the specified object.
     */
    protected PortView createPortView(JGraph g, CellMapper cm, Object cell) {
        return new MyPortView(g, cm, (MyPort) cell);
    }

    private boolean isPrimitiveVertex(Object cell) {
        return cell instanceof PrimitiveVertex;
    }

    private boolean isCompositeVertex(Object cell) {
        return cell instanceof CompositeVertex;
    }

    /**
     * Constructs a PortView view for the specified object.
     */
    protected VertexView createVertexView(JGraph g, CellMapper cm, Object cell) {
        if (isCompositeVertex(cell))
            return new CompositeVertexView(g, cm, (CompositeVertex) cell);
        else if (isPrimitiveVertex(cell))
            return new PrimitiveVertexView(g, cm, (PrimitiveVertex) cell);
        else
            return null;
    }

    public CellView createView(JGraph graph, CellMapper mapper, Object cell) {
        CellView view = null;
        if (graph.getModel().isPort(cell))
            view = createPortView(graph, mapper, cell);
        else if (graph.getModel().isEdge(cell))
            view = createEdgeView(graph, mapper, cell);
        else
            view = createVertexView(graph, mapper, cell);
        mapper.putMapping(cell, view);
        view.refresh(true);
        view.update();
        return view;
    }

}