/*====================================================================

 Objectweb Explorer Framework
 Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
 Contact: openccm@objectweb.org

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 USA

 Initial developer(s): Latitia Lafeuille.
 Contributor(s): ______________________________________.

 ====================================================================*/
package org.objectweb.util.explorer.swing.graph;

import java.awt.Dimension;
import java.util.ArrayList;
import java.util.List;

import org.jgraph.graph.DefaultGraphCell;

/**
 * 
 * @version 0.2
 */
public class PrimitiveVertex extends DefaultGraphCell {

    /** The component relative to the vertex */
    private Object component;

    /** The vertex ports list */
    private ArrayList ports;

    /** The vertex client ports list */
    private ArrayList clientPorts;

    /** The vertex server ports list */
    private ArrayList serverPorts;

    /** The vertex controller ports list */
    private ArrayList controllerPorts;

    /** The vertex name */
    private String name;

    /** The graph that contain the vertex */
    private Graph graph;

    /** Boolean tht indicates if the component is started or not */
    private boolean isStarted;

    /** Boolean that indicates if the component is shared or not */
    private boolean isShared;

    /** Number of the specific port */
    private int numberServer, numberClient, numberControler,
            numberInternalServer, numberInternalClient;

    /** The type of the vertex : primitive/composite */
    transient String type;

    /** The size of the vertex */
    private Dimension size = new Dimension();

    /** The size of the superComposite */
    private Dimension compositeSize = new Dimension();

    /** Boolean that indicate if the controllers must be displayed or not */
    private boolean controllersDisplay;

    private boolean isFirstPaint;

    public PrimitiveVertex(Object component, String name, Graph graph,
            Dimension compositeSize, boolean controllersDisplay,
            boolean isStarted, boolean isShared) {
        super(name);
        this.name = name;
        this.graph = graph;
        this.controllersDisplay = controllersDisplay;
        this.component = component;
        this.type = VertexType.PRIMITIF_VERTEX;
        this.isStarted = isStarted;
        this.isShared = isShared;
        ports = new ArrayList();
        clientPorts = new ArrayList();
        serverPorts = new ArrayList();
        controllerPorts = new ArrayList();
        numberServer = 0;
        numberClient = 0;
        numberControler = 0;
        numberInternalServer = 0;
        numberInternalClient = 0;
        setCompositeSize(compositeSize);
        isFirstPaint = true;
    }

	/**
	 * Add a port on the vertex
	 * @param itf, the interface relative to the port to add
	 * @param name, the name of the port
	 * @param type, the type of the port
	 */
    public void addPort(Object itf, String name, String type) {
        MyPort p = null;
        if ((type.equals(PortType.CLIENT_PORT))
                || (type.equals(PortType.COLLECTION_PORT))) {
            numberClient++;
            p = new MyPort(itf, name, type, numberClient, false);
        } else if (type.equals(PortType.SERVER_PORT)) {
            numberServer++;
            p = new MyPort(itf, name, type, numberServer, false);
        } else if (type.equals(PortType.CONTROLLER_PORT)) {
            numberControler++;
            p = new MyPort(itf, name, type, numberControler, false);
        }
        super.add(p);
        ports.add(p);
    }

    public boolean isFirstPaint() {
        return isFirstPaint;
    }

    public void setFirstPaint(boolean isFirstPaint) {
        this.isFirstPaint = isFirstPaint;
    }

    public boolean isShared() {
        return isShared;
    }

    public void setShared(boolean isShared) {
        this.isShared = isShared;
    }

    public boolean isStarted() {
        return isStarted;
    }

    public void setStarted(boolean isStarted) {
        this.isStarted = isStarted;
    }

    public Object getComponent() {
        return component;
    }

    public boolean getControllerDisplay() {
        return controllersDisplay;
    }

    public void setControllerDisplay(boolean controllersDisplay) {
        this.controllersDisplay = controllersDisplay;
    }

    public ArrayList getListPorts() {
        return ports;
    }

	/**
	 * Research a port with his name
	 * @param name, the name of the port researched
	 * @return the port researched
	 */
    public MyPort getPort(String name) {
        for (int i = 0; i < ports.size(); i++) {
            if (((MyPort) ports.get(i)).getName().equals(name))
                return (MyPort) ports.get(i);
        }
        return null;
    }

	/**
	 * Research a port with his name, his type and his number.
	 * @param name, the name of the port
	 * @param type, the type of the port
	 * @param number, the number of the port
	 * @return the port researched
	 */
    public MyPort getPort(String name, String type, int number) {
        List ports;
        if (type.equals(PortType.CLIENT_PORT)) {
            ports = getListClientPorts();
        } else if (type.equals(PortType.SERVER_PORT)) {
            ports = getListServerPorts();
        } else
            ports = getListControllerPorts();

        for (int i = 0; i < ports.size(); i++) {
            if ((((MyPort) ports.get(i)).getName().equals(name))
                    && (((MyPort) ports.get(i)).getNumber() == number))
                return (MyPort) ports.get(i);
        }
        return null;
    }

    public String getName() {
        return name;
    }

    public String getType() {
        return type;
    }

	/**
	 * @param port
	 * @return the width of the label of the port
	 */
    public int getPortLabelWidth(MyPort port) {
        return port.getName().length() * PortGraphicsInterface.PORT_NAME_WIDTH;
    }

	/**
	 * Return the width of a line of a vertex.
	 * The lines of a vertex are determined by the number of client/server ports.
	 * The width is determinate with the width of the ports label and the width of the vertex label 
	 * @param line, the line in the vertex
	 * @return the width of the line
	 */
    private int getMinWidht(int line) {
        int minWidth = 0;
        List servers = getListServerPorts();
        List clients = getListClientPorts();
        int serversLength = servers.size();
        int clientsLength = clients.size();
        if ((serversLength > line) && (clientsLength > line)) {
            MyPort server = (MyPort) servers.get(line);
            MyPort client = (MyPort) clients.get(line);
            if (getPortLabelWidth(server) > getPortLabelWidth(client))
                minWidth = getPortLabelWidth(server) * 2;
            else
                minWidth = getPortLabelWidth(client) * 2;
        } else if (clientsLength > line) {
            MyPort client = (MyPort) clients.get(line);
            minWidth = getPortLabelWidth(client) * 2;
        } else if (serversLength > line){
            MyPort server = (MyPort) servers.get(line);
            minWidth = getPortLabelWidth(server) * 2;
        }
        return minWidth + getName().length()*VertexGraphicsInterface.PRIMITIVE_NAME_WIDTH;
    }

    /**
     * Returns the minimum width that the vertex must have to draw correctly his
     * label and his ports label
     * @return the minimum width that the vertex must have
     */
    public int getMinWidth() {
        int minWidth = 0;
        List servers = getListServerPorts();
        List clients = getListClientPorts();
        int serversLength = servers.size();
        int clientsLength = clients.size();
        if(serversLength>clientsLength){
            for(int i=0;i<serversLength;i++){
                if(minWidth<getMinWidht(i))
                    minWidth=getMinWidht(i);
            }
        }
        else{
            for(int i=0;i<clientsLength;i++){
                if(minWidth<getMinWidht(i))
                    minWidth=getMinWidht(i);
            }
        }
       
        return minWidth;
    }

    public ArrayList getListServerPorts() {
        if (serverPorts.size() == 0) {
            List ports = getListPorts();
            for (int i = 0; i < ports.size(); i++) {
                if (((MyPort) ports.get(i)).getType().equals(
                        PortType.SERVER_PORT))
                    serverPorts.add(ports.get(i));
            }
        }
        return serverPorts;
    }

    public ArrayList getListClientPorts() {
        if (clientPorts.size() == 0) {
            List ports = getListPorts();
            for (int i = 0; i < ports.size(); i++) {
                if ((((MyPort) ports.get(i)).getType()
                        .equals(PortType.CLIENT_PORT))
                        || (((MyPort) ports.get(i)).getType()
                                .equals(PortType.COLLECTION_PORT)))
                    clientPorts.add(ports.get(i));
            }
        }
        return clientPorts;
    }

    public ArrayList getListControllerPorts() {
        if (controllerPorts.size() == 0) {
            List ports = getListPorts();
            for (int i = 0; i < ports.size(); i++) {
                if (((MyPort) ports.get(i)).getType().equals(
                        PortType.CONTROLLER_PORT))
                    controllerPorts.add(ports.get(i));
            }
        }
        return controllerPorts;
    }

    /**
     * Return the size of the vertex calculated in relation with the number of
     * ports
     */
    public Dimension getSize() {
        VertexGraphicsInterface vg = Graph.getVertexGraphics();
            PortGraphicsInterface pg = Graph.getPortGraphics();
            int portsServer = getListServerPorts().size();
            int portsClients = getListClientPorts().size();
            int portsControl = getListControllerPorts().size();
            if (size.height < portsServer
                    * PortGraphicsInterface.PORT_NAME_LENGHT) {
                size.height = portsServer
                        * PortGraphicsInterface.PORT_NAME_LENGHT
                        + PortGraphicsInterface.FIRST_PORT_Y;
            }
            if (size.height < portsClients
                    * PortGraphicsInterface.PORT_NAME_LENGHT) {
                size.height = portsClients
                        * PortGraphicsInterface.PORT_NAME_LENGHT
                        + PortGraphicsInterface.FIRST_PORT_Y;
            }
            if (size.width < portsControl
                    * PortGraphicsInterface.CONTROLLER_PORT_WIDTH) {
                size.width = portsControl
                        * PortGraphicsInterface.CONTROLLER_PORT_WIDTH;
            }
            int minWidth=getMinWidth();
            if(minWidth>VertexGraphicsInterface.VERTEX_MAX_WIDTH)
                minWidth=VertexGraphicsInterface.VERTEX_MAX_WIDTH;
            if (size.width < minWidth)
                size.width = minWidth;
            if ((size.height == 0)
                    || (size.height < vg.getDefaultSize().height))
                size.height = vg.getDefaultSize().height;
            if ((size.width == 0) || (size.width < vg.getDefaultSize().width))
                size.width = vg.getDefaultSize().width;
        return size;
    }

    public void setSize(Dimension d) {
        size.width = d.width;
        size.height = d.height;
    }

    public Dimension getCompositeSize() {
        return compositeSize;
    }

    public void setCompositeSize(Dimension d) {
        compositeSize.width = d.width;
        compositeSize.height = d.height;
    }

}