#include <microccm_CruiseControl_executor.h>

#include <threading_utils.h>

#include <iostream>

using std::cerr;
using std::endl;

class CruiseControl_impl;

// -------------------------------------------------------

class BrakeInterface_impl : public CCM_BrakeInterface
{
public:

  BrakeInterface_impl( CruiseControl_impl* cc_impl ) : cc_impl_( cc_impl ) 
  {
    brake_state_.set( released );
  }

  void do_it( BrakeAction ba );

private:

  friend class EngineInterface_impl;
  friend class CruiseControlInterface_impl;

  threading::protected_resource< BrakeAction > brake_state_;

  CruiseControl_impl* cc_impl_;
};

// -------------------------------------------------------

class EngineInterface_impl : public CCM_EngineInterface
{
public:

  EngineInterface_impl( CruiseControl_impl* cc_impl ) : cc_impl_( cc_impl ) 
  { 
    engine_state_.set( off );
  }

  void do_it( EngineAction ea );

private:

  friend class CruiseControlInterface_impl;

  threading::protected_resource< EngineAction > engine_state_;

  CruiseControl_impl* cc_impl_;
};

// --------------------------------------------------------

class CruiseControlInterface_impl : public CCM_CruiseControlInterface
{
public:

  CruiseControlInterface_impl( CruiseControl_impl* cc_impl ) : cc_impl_( cc_impl ) { }
  void do_it( CruiseAction ca );

private:

  friend class BrakeInterface_impl;

  threading::protected_resource< CruiseAction > cruise_state_;
  CruiseControl_impl* cc_impl_;
};

// ---------------------------------------------------------

class CruiseControl_impl : public CCM_CruiseControl,
			   public Components::SessionComponent
{
public:

  CruiseControl_impl( );
  ~CruiseControl_impl( );

  CCM_BrakeInterface_ptr get_brk_interface();
  CCM_EngineInterface_ptr get_eng_interface();
  CCM_CruiseControlInterface_ptr get_cc_interface();

  void configuration_complete() throw( Components::InvalidConfiguration );
  void set_session_context( Components::SessionContext_ptr the_session_context )
    throw( Components::CCMException );

private:

  pthread_t the_thread_;

  CCM_CruiseControl_Context_ptr ctx_;

  friend class BrakeInterface_impl;
  friend class EngineInterface_impl;
  friend class CruiseControlInterface_impl;

  BrakeInterface_impl* the_brk_intf_;
  EngineInterface_impl* the_eng_intf_;
  CruiseControlInterface_impl* cc_intf_;
};

// -----------------------------------------------------------------------------

CruiseControl_impl::CruiseControl_impl()
{
  cerr << __PRETTY_FUNCTION__ << " BEGIN" << endl;
  the_brk_intf_ = new BrakeInterface_impl( this );
  the_eng_intf_ = new EngineInterface_impl( this );
  cc_intf_ = new CruiseControlInterface_impl( this );
  cerr << __PRETTY_FUNCTION__ << " END " << endl;
}

CruiseControl_impl::~CruiseControl_impl()
{
  pthread_cancel( the_thread_ );

  delete the_brk_intf_;
  delete the_eng_intf_;
  delete cc_intf_;
}

void 
CruiseControl_impl::configuration_complete() throw( Components::InvalidConfiguration )
{
}


void
CruiseControl_impl::set_session_context( Components::SessionContext_ptr ctx )
  throw( Components::CCMException )
{
  ctx_ = CCM_CruiseControl_Context::_narrow( ctx );
  cerr << __PRETTY_FUNCTION__ << " Ok" << endl;
}

CCM_BrakeInterface_ptr CruiseControl_impl::get_brk_interface()
{
  return the_brk_intf_;
}

CCM_EngineInterface_ptr CruiseControl_impl::get_eng_interface()
{
  return the_eng_intf_;
}

CCM_CruiseControlInterface_ptr CruiseControl_impl::get_cc_interface()
{
  return cc_intf_;
}

// ----------------------------------------------------------------------------

void BrakeInterface_impl::do_it( BrakeAction a )
{
  if( a == pressed )
    {
      if( cc_impl_->cc_intf_->cruise_state_.get( ) == cruise )
	{
	  cc_impl_->cc_intf_->do_it( cruise_off );
	}

      if( cc_impl_->cc_intf_->cruise_state_.get( ) == accel )
	{
	  cc_impl_->cc_intf_->do_it( accel_off );
	}
    }

  brake_state_.set( a );
}

void EngineInterface_impl::do_it( EngineAction a )
{
  engine_state_.set( a );
}

void CruiseControlInterface_impl::do_it( CruiseAction a )
{
  if( cc_impl_->the_brk_intf_->brake_state_.get() == pressed )
    {
      cerr << "Must release brake in order to do some cruising action .." << endl;
      return;
    }

  if( cc_impl_->the_eng_intf_->engine_state_.get() == off )
    {
      cerr << "Must turn on engine in order to do some cruising action .." << endl;
      return;
    }

  // -------- acceleration --------

  if( a == accel )
    {
      // if we are currently braking, do nothing
      // else do acceleration
      if( cruise_state_.get() == cruise )
	{
	  cc_impl_->ctx_->get_connection_cruiser_intf()->disable_maintain_speed();
	}

      cc_impl_->ctx_->get_connection_accel_intf()->enable_increase_speed();
    }

  if( a == accel_off )
    {
      if( cruise_state_.get() != accel )
	{
	  cerr << "Trying to accell_off but you were not accelerating" << endl;
	  return;
	}

      cc_impl_->ctx_->get_connection_accel_intf()->disable_increase_speed();
    }

  // --------- cruising -------------

  if( a == cruise )
    {
      if( cruise_state_.get() == accel )
	{
	  cc_impl_->ctx_->get_connection_accel_intf()->disable_increase_speed();
	}

      // i.e maintain speed we have reached
      // set DesiredSpeed to be CurrentSpeed

      CORBA::Float current_speed 
	= cc_impl_->ctx_->get_connection_current_speed()->speed();

      cc_impl_->ctx_->get_connection_desired_speed()->speed( current_speed );

      cc_impl_->ctx_->get_connection_cruiser_intf()->enable_maintain_speed();
    }

  if( a == cruise_off )
    {
      if( cruise_state_.get() != cruise )
	{
	  cerr << "Can't stop cruising because you were not cruising" << endl;
	  exit( 1 );
	}

      // i.e stop maintaining desired speed
      cc_impl_->ctx_->get_connection_cruiser_intf()->disable_maintain_speed();
    }

  if( a == resume )
    {
      // don't know what to do here ...
    }

  cruise_state_.set( a );
}

// -----------------------------------------------------------------------------

class CruiseControlHome_impl : public CCM_CruiseControlHome
{
public:

  Components::EnterpriseComponent_ptr create() throw( Components::CCMException );
};

Components::EnterpriseComponent_ptr CruiseControlHome_impl::create()
  throw( Components::CCMException )
{
  cerr << __PRETTY_FUNCTION__ << endl;
  return new CruiseControl_impl;
}

// ------------------------------------------------------------------------------

extern "C"
{
  Components::HomeExecutorBase* create_home_exec()
  {
    cerr << __PRETTY_FUNCTION__ << endl;
    return new CruiseControlHome_impl;
  }
}

// ------------------------------------------------------------------------------

