#ifndef IDL_to_cpp_mapping_aux_h
#define IDL_to_cpp_mapping_aux_h


/** Class designed for methods arguments conversion to CORBA C++ mapping. Abstract class 
 * allowing to convert method arguments to CORBA C++ mapping used to  
 * generate cpp mapping of interfaces. Classes inherited from this
 * one are dedicated to each type of argument: 
 * basic, enum, objref, struct, wstring, any, sequence.
 */
class Cpp_mapping_arg_conversion
{
 public:
  /** This virtual method convert argument passing its name and its mode. */ 
  virtual string convert( string name, arg_passing_mode apm ) = 0;
};


/**  Class designed for basic method's arguments conversion to CORBA C++ mapping.
 * Class allowing to convert basic method's arguments(like: short, long,
 * longlong, etc.) in order to generate the corresponding CORBA C++ mapping 
 * of interfaces.
 */
class Cpp_mapping_basic_arg_conv : public Cpp_mapping_arg_conversion
{
 public:
  Cpp_mapping_basic_arg_conv( string corba_corresp_name );

  /** This method implements Cpp_mapping_arg_conversion::convert method to convert 
   * basic argument passing its name and its mode.*/ 
  string convert( string name, arg_passing_mode apm );

 private:
  string corba_corresp_name_;
};



/**  Class designed for enum method's arguments conversion to CORBA C++ mapping.
 * Class allowing to convert basic method's arguments(like: short, long,
 * longlong, etc.) in order to generate the corresponding CORBA C++ mapping 
 * of interfaces.
 */
class Cpp_mapping_enum_conv : public Cpp_mapping_arg_conversion
{
 public:

  /** This method implements Cpp_mapping_arg_conversion::convert method to convert 
   * enum argument passing its name and its mode.*/ 
  string convert( string name, arg_passing_mode apm );
};



/**  Class designed for object reference method's arguments conversion to CORBA C++ mapping.
 * Class allowing to convert object reference method's arguments(like: short, long,
 * longlong, etc.) in order to generate the corresponding CORBA C++ mapping 
 * of interfaces.
 */
class Cpp_mapping_objref_conv : public Cpp_mapping_arg_conversion
{
 public:
  
  /** This method implements Cpp_mapping_arg_conversion::convert method to convert 
   * object reference argument passing its name and its mode.*/ 
  string convert( string name, arg_passing_mode apm );
};



/**  Class designed for struct method's arguments conversion to CORBA C++ mapping.
 * This class allows to convert struct method's arguments in order to generate 
 * the corresponding CORBA C++ mapping of interfaces.
 */
class Cpp_mapping_struct_conv : public Cpp_mapping_arg_conversion
{
 public:
   /** This method implements Cpp_mapping_arg_conversion::convert method to convert 
    *  struct argument passing its name and its mode.*/ 
  string convert( string name, arg_passing_mode apm );

 private:
  bool is_variable_size( const char* name_of_struct );
};




/**  Class designed for wstring method's arguments conversion to CORBA C++ mapping.
 * This class allows to convert wtring method's arguments in order to generate 
 * the corresponding CORBA C++ mapping of interfaces.
 */
class Cpp_mapping_wstring_string_conv : public Cpp_mapping_arg_conversion
{
 public:
   /** This method implements Cpp_mapping_arg_conversion::convert method to convert 
    * wstring argument passing its name and its mode.*/ 
  Cpp_mapping_wstring_string_conv( string wstring_or_string );

  string convert( string name, arg_passing_mode apm );

 private:

  string corresponding_base_name_;
};

/**  Class designed for sequence method's arguments conversion to CORBA C++ mapping.
 * This class allows to convert wtring method's arguments in order to generate 
 * the corresponding CORBA C++ mapping of interfaces.
 */
class Cpp_mapping_sequence_conv : public Cpp_mapping_arg_conversion
{
 public:
  /** This method implements Cpp_mapping_arg_conversion::convert method to convert 
   * sequence argument passing its name and its mode.*/ 
  string convert( string name, arg_passing_mode apm );
};



/**  Class designed for type any method's arguments conversion to CORBA C++ mapping.
 * This class allows to convert method's arguments of type any in order to generate 
 * the corresponding CORBA C++ mapping of interfaces.
 */
class Cpp_mapping_any_conv : public Cpp_mapping_arg_conversion
{
 public:
  /** This method implements Cpp_mapping_arg_conversion::convert method to convert 
   * any argument passing its name and its mode.*/ 
  string convert( string name, arg_passing_mode apm );
};



inline bool is_basic_type( CORBA::TCKind tckind )
{
  if( tckind == CORBA::tk_void) return true;
  if( tckind == CORBA::tk_short ) return true;
  if( tckind == CORBA::tk_long ) return true;
  if( tckind == CORBA::tk_longlong ) return true;
  if( tckind == CORBA::tk_ushort ) return true;
  if( tckind == CORBA::tk_ulong  ) return true;
  if( tckind == CORBA::tk_ulonglong ) return true;
  if( tckind == CORBA::tk_float ) return true;
  if( tckind == CORBA::tk_double ) return true;
  if( tckind == CORBA::tk_longdouble ) return true;
  if( tckind == CORBA::tk_boolean ) return true;
  if( tckind == CORBA::tk_char ) return true;
  if( tckind == CORBA::tk_wchar ) return true;
  if( tckind == CORBA::tk_octet ) return true;

  if( tckind == CORBA::tk_string ) return true;
  if( tckind == CORBA::tk_wstring ) return true;

  if( tckind == CORBA::tk_any ) return true;

  return false;
}

#endif // IDL_to_cpp_mapping_aux_h
