/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.corba.generator.cif_idl.lib;

// Package dependencies.
import org.objectweb.openccm.ast.api.ModuleDecl;
import org.objectweb.openccm.ast.api.InterfaceList;
import org.objectweb.openccm.ast.api.InterfaceDecl;
import org.objectweb.openccm.ast.api.InterfacePortDecl;
import org.objectweb.openccm.ast.api.EventPortDecl;
import org.objectweb.openccm.ast.api.ComponentDecl;
import org.objectweb.openccm.ast.api.SegmentDecl;
import org.objectweb.openccm.ast.api.NativeDecl;
import org.objectweb.openccm.ast.api.OperationDecl;

/**
 * This class generates CIF IDL for an OMG CIDL Segment.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 *
 * @version 0.1
 */

public class SegmentMapping
  implements org.objectweb.corba.generator.cif_idl.api.SegmentMapping
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The segment to map. */
    private SegmentDecl seg_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public SegmentMapping(SegmentDecl seg)
    {
        // Init internal states
        seg_ = seg;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Create Segment Base CIF interface mapping : CIF_<segment_name>.
     *
     * @param module - The module to declare CIF mapping in.
     * @param seg_base_mapping - The Segment base CIF interface mapping.
     * @param comp - The binding component.
     *
     * @return The segment CIF mapping.
     */
    protected InterfaceDecl
    create_seg_mapping(ModuleDecl module,
                       InterfaceDecl seg_base_mapping,
                       ComponentDecl comp)
    {
        InterfaceDecl seg_mapping = null;
        InterfaceList list = null;
        String[] facets = null;
        String[] sinks = null;

        seg_mapping = module.declareLocalInterface( "CIF_" + seg_.getName() );
        list = seg_mapping.getInheritedInterfaceList();

        // Add the CIF_SegmentBase interface
        list.add(seg_base_mapping);

        // Add the CCM_<providing_object_name> interface
        // for each facet implemented
        facets = seg_.getFacetList().getStrings();
        for (int i=0; i<facets.length; i++)
        {
            InterfacePortDecl itf_port = null;
            ComponentDecl current = null;

            current = comp;
            while ((current != null) && (itf_port == null))
            {
                itf_port = (InterfacePortDecl) current.lookup(facets[i]);
                // Search facet declaration in the base component
                current = comp.getBaseComponent();
            }
            list.add( itf_port.getLocalMapping() );
        }

        // Add the CCM_<providing_object_name> interface
        // For each sink implemented
        sinks = seg_.getSinkList().getStrings();
        for (int i=0; i<sinks.length; i++)
        {
            EventPortDecl event_port = null;
            ComponentDecl current = null;

            current = comp;
            while ((current != null) && (event_port == null))
            {
                event_port = (EventPortDecl) current.lookup(sinks[i]);
                // Search sink declaration in the base component
                current = comp.getBaseComponent();
            }
            list.add( event_port.getEvent().getLocalConsumerMapping() );
        }

        // Create the mapping
        seg_mapping.create();

        return seg_mapping;
    }

    /**
     * Add Segment Base CIF interface mapping contents.
     *
     * @param seg_mapping - The Segment CIF interface mapping.
     */
    protected void
    add_seg_mapping_contents(InterfaceDecl seg_mapping)
    {
        /** Create operations for persistence if needed **/
        if (seg_.getAbstractStorageHome() != null)
        {
            NativeDecl ast = null,
                       ash = null;
            OperationDecl op = null;

            ash = seg_.getAbstractStorageHome().getMapping();
            ast = seg_.getAbstractStorageHome().getAbstractStorageType().getMapping();

            op = seg_mapping.startOperation("get_state");
            op.setType(ast);
            op.create();
            // Create an operation to get home ?
        }
    }

    // ==================================================================
    //
    //  Public methods for org.objectweb.openccm.generator.cif_idl.api.SegmentMapping
    //
    // ==================================================================

    /**
     * Declare CIF interfaces into Interface Repository.
     *
     * @param module - The module to declare CIF mapping in.
     * @param seg_base_mapping - The Segment CIF interface mapping.
     * @param comp - The binding component.
     */
    public void
    declareCIF(ModuleDecl module,
               InterfaceDecl seg_base_mapping,
               ComponentDecl comp)
    {
        InterfaceDecl seg_mapping = null;

        seg_mapping = create_seg_mapping(module,
                                         seg_base_mapping,
                                         comp);
        add_seg_mapping_contents(seg_mapping);
    }

    // ==================================================================
    //
    //  Public methods.
    //
    // ==================================================================

}
