/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Hameau Fabien.
Contributor(s): ______________________________________.

====================================================================*/
package org.objectweb.corba.generator.metainformation.lib;


//Package dependencies.

import org.objectweb.openccm.ast.api.*;
import org.objectweb.openccm.generator.java.core.lib.*;
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
import org.objectweb.openccm.generator.translator.idl2java.lib.CommonTranslator;
import org.objectweb.openccm.generator.common.lib.FileManager;
import org.objectweb.openccm.generator.common.lib.GenerationException;

/**
 * This generator takes a IDL3 declaration from
 * the repository and generate appriopriate files
 * to support MetaInformation implementation.
 * 
 * MI is an implementation of D2.1 specification
 * (COACH) about static and dynamic CCM concept 
 * information handling.   
 *
 * @author <a href="mailto:hameau@lifl.fr">Hameau</a>
 *
 * @version 0.1
 */

public class IDL_JavaMIGenerator
extends org.objectweb.openccm.generator.common.lib.GeneratorBase
implements org.objectweb.corba.generator.metainformation.api.IDL_JavaMIGenerator
{
  // ==================================================================
  //
  // Internal states.
  //
  // ==================================================================
  
  /** Java Repository */
  private Repository _repository; 
  private Scope CCMObj_scope;
  /** utility to convert types in Java */
  public CommonTranslator _translator;
  private AST _ast;
  private String  _scope_name;

  /**
   * 
   */
  
  // ==================================================================
  //
  // Constructors.
  //
  // ==================================================================
   
  /**
   * This constructor initialize the IDL3->MI compiler
   * with 
   * - the concerned AST
   * - the creation of a new 'JavaTranslator' to generate 
   *   java source code, and retrieve the scope of
   * - CCMObject to add some IDL3-IDL2 artefact mapping ( such
   *   as equivalent interface for home and component)...
   *
   * @param ast : the AST associated to the IDL3 declaration
   */

  public IDL_JavaMIGenerator(AST ast)
  {
    super(ast);
    _repository = null;
    _translator = new CommonTranslator();
    CCMObj_scope = null;
    try
    {
      CCMObj_scope = (Scope) ast.lookup("::Components::CCMObject");
    }
    catch (ClassCastException e)
    {
      System.err.println("Declaration of CCMObject not found!");
    }
    
    _ast = ast;
  }

 
  // ==================================================================
  //
  // Internal methods.
  //
  // ==================================================================
  
  /**
   * mapHomeMI
   * 
   * generates the appropriate type MI class for a given home
   * declaration. Map the home type MI in a class
   * 
   * @param _home_decl - the home declaration
   * @param pkg_name   - the home package name
   * 
   */
  
  private void mapHomeMI(HomeDecl _home_decl, String pkg_name)
  {
    int i;
    
    ClassObject clazz       = null;
    ConstructorObject _cstr = null;
    
    // naming the class 
        
    String home_mi_class_name = _home_decl.getName() + "_homeMI"; // name of the home MI class
    clazz = new ClassObjectImpl(home_mi_class_name);    // create its representation
    
    // filling the class representation
    
    // some comments on generated class
    clazz.addComment("This class represents the home type MetaInformation for the given " + _home_decl.getName());
    // class is public
    clazz.setModifier(ModifierKindImpl.mk_public);
    // set the package name for home MI class
    clazz.setPackage(pkg_name);           
    // this class inherits from the home type MI implementation         
    clazz.addInheritedObject("org.objectweb.openccm.Containers.MetaInformation.HomeTypeImpl");
    
    clazz.addImport("org.objectweb.openccm.Containers.MetaInformation.*");
    // add specific home declaration elements
    // in the constructor
    
    _cstr = new ConstructorObjectImpl();
    _cstr.getImpl().setMacro("HOME_MI_CONSTRUCTOR"); // set velocity macro
    
    // set type MI properties for the given home
    
    _cstr.getImpl().addContextValue("name", _home_decl.getName()); // type name
    _cstr.getImpl().addContextValue("rep_id", _home_decl.getId()); // type repository ID
    _cstr.getImpl().addContextValue("type_instance_factory_entrypt",
                                    pkg_name + "."+_home_decl.getName() + "CCM");
                                     
    //////////////////////                               
    // managed components
    //////////////////////
    
    _cstr.getImpl().addContextValue("component_class_name", 
                                    _home_decl.getManagedComponent().getName());
    // managed attributes 
    
    Declaration[] _home_atts = _home_decl.getContents(true,DeclarationKind.dk_attribute);
    if(_home_atts.length != 0)
    {
      _cstr.getImpl().addContextValue("att_list", _home_atts);
    }
    
    ///////////////////////
    // supported interfaces
    ///////////////////////
    
    // get the supported interface list 
    InterfaceDecl[] intfs = _home_decl.getSupportedInterfaceList().getInterfaces();
    // put a macro to add the supported intf to the home
    if (intfs.length != 0)
    {
      _cstr.getImpl().addContextValue("interface_list", intfs);
      // set value for given
      // check for imported interfaces
      for (int j = 0; j < intfs.length; j++)
      {
        // declaration from another scope ?
        if (!(intfs[j].getAbsoluteName().startsWith(_scope_name)))
        {
          // generate the associated file 
          Declaration _imported = _ast.lookup(intfs[j].getAbsoluteName());
          mapInterfaceMI((InterfaceDecl) _imported, pkg_name);
        }
      }
    }
 
    // The contructor is now well created, and can be added to the class
    // definition
    
    clazz.addConstructor(_cstr);
    
    // static entrypt for call creation
    
    MethodObjectImpl _met = new MethodObjectImpl();
    
    _met.setName("create_type_MI");
    _met.setReturnType("HomeType");
    _met.setModifier(ModifierKindImpl.mk_public);
    _met.setStatic(true);
    _met.addComment("static entrypt for container and installation");
 
    _met.getImpl().setMacro("STATIC_CALL_HOME_MI");
    _met.getImpl().addContextValue("classname",_home_decl.getName() + "_homeMI");

    clazz.addMethod(_met);
    // No more properties are set for the home type MI class, it can be
    // added to the java repository for code generation
    
    _repository.addObject(clazz);         
    
    // The Home also has an equivalent interface !
    // As a component can offer a receptacle that reference a home
    // the creation of the home equivalent interface is delegated 
    // to an other class.
    
    // naming the class 
    
    String home_equv_itf_mi_class_name = _home_decl.getName() + "_interfaceMI"; // name of the home MI class
    clazz = new ClassObjectImpl(home_equv_itf_mi_class_name);    // create its representation

    // filling the class representation
 
    // some comments on generated class
    clazz.addComment("This class represents the home equivalent interface type MetaInformation ");
    clazz.addComment("for the given " + _home_decl.getName());
    // class is public
    clazz.setModifier(ModifierKindImpl.mk_public);
    // set the package name for home MI class
    clazz.setPackage(pkg_name);           
    // this class inherits from the home type MI implementation         
    clazz.addInheritedObject("org.objectweb.openccm.Containers.MetaInformation.InterfaceTypeImpl");

    clazz.addImport("org.objectweb.openccm.Containers.MetaInformation.*");
    // add specific home declaration elements
    // in the constructor

    _cstr = new ConstructorObjectImpl();
    _cstr.getImpl().setMacro("HOME_EQUIV_ITF"); // set velocity macro the given home equivalent interface
    AttributeObject param1 = new AttributeObjectImpl();
    param1.setName("type_ref");
    param1.setType("TypeImpl");
    _cstr.addParameter(param1);
    AttributeObject param2 = new AttributeObjectImpl();
    param2.setName("type_ki");
    param2.setType("type_kind");
    _cstr.addParameter(param2);
  
    // Fill the specific Home equivalent interface Type MI
  
    _cstr.getImpl().addContextValue("name", "???"); //   TODO : find the good name
    _cstr.getImpl().addContextValue("rep_id", "???"); // TODO : find the good repository ID
    _cstr.getImpl().addContextValue("type_instance_factory_entrypt",
                                    "???"); // TODO : find the good entrypt

    // add the specific operations to the interface
    
    Declaration[] _equiv_itf_decl = _home_decl
                                       .getClientMapping()
                                       .getContents(false, DeclarationKind.dk_operation);
        
    _cstr.getImpl().addContextValue("opt_list",_equiv_itf_decl);
      
    clazz.addConstructor(_cstr);

    // the class is finished and ready to be generated

    _repository.addObject(clazz);         
  };
  
  /**
   * mapComponentMI
   * 
   * generates the appropriate type MI class for a given 
   * component declaration. Map the componnet type MI 
   * in a class
   * 
   * @param _component_decl - the home declaration
   * @param pkg_name   - the home package name
   * 
   */
    
  private void mapComponentMI(ComponentDecl decl, String pkg_name)
  {
    ClassObject       clazz = null;
    ConstructorObject ct    = null;
    
    // retrieve the name and create the class obj
        
    String class_name = new String(decl.getName() + "_componentMI");
    clazz = new ClassObjectImpl(class_name);
    
    // filling the class representation
    
    clazz.addImport("org.objectweb.openccm.Containers.MetaInformation.*");
    
    clazz.addComment("This class is the component type MetaInformation");
    clazz.addComment("for the given component. It gathers static properties");
    clazz.addComment("of CCM concepts managed by the related component");
    clazz.setModifier(ModifierKindImpl.mk_public); // class is public
    clazz.setPackage(pkg_name);                    // set the package name           
    clazz.addInheritedObject("org.objectweb.openccm.Containers.MetaInformation.ComponentTypeImpl");
    
    // add the specific code for managed CCM concept for 
    // a given component, from its IDL3 declaration
    
    ct = new ConstructorObjectImpl();
    ct.getImpl().setMacro("COMPONENT_MI_CONSTRUCTOR"); // set velocity macro
    AttributeObject param = new AttributeObjectImpl();
    param.setName("home_type");
    param.setType("HomeTypeImpl");
    ct.addParameter(param);
    
    // for static component value
    
    ct.getImpl().addContextValue("name", decl.getName());
    ct.getImpl().addContextValue("rep_id", decl.getId());
    ct.getImpl().addContextValue("type_instance_factory_entrypt", "???");
    
    //  about inherited interface
    // concerns facet, receptacles event source and sink
    
    ComponentDecl _tmp_comp = decl.getBaseComponent();
    
    ///////////////////////////
    // for supported interfaces
    ///////////////////////////
    
    // get supported interface(s) list
    InterfaceDecl[] _intf_decl = decl.getSupportedInterfaceList().getInterfaces();
    // put a macro adding this interface to the suported interfaces        
    if(_intf_decl.length != 0)
    { 
      ct.getImpl().addContextValue("int_list", _intf_decl);
    }
    // check for imported interfaces 
    for (int i = 0; i < _intf_decl.length; i++)
    {
      // declaration from another scope ?
      if(!(_intf_decl[i].getAbsoluteName().startsWith(_scope_name)))
      {
        // generate the associated file 
        Declaration _imported = _ast.lookup(_intf_decl[i].getAbsoluteName());
        mapInterfaceMI((InterfaceDecl) _imported, pkg_name);
      }
    }
    
    ///////////////////////////
    // for managed attribute(s)
    ///////////////////////////
    
    Declaration[] _att_decl = decl.getContents(false,DeclarationKind.dk_attribute);
    
    if(_att_decl.length != 0)
    {
      ct.getImpl().addContextValue("att_list", _att_decl);
    }
    
    /////////////////////
    // facet port type MI
    /////////////////////
    
    // get primary eclaration 
    
    Declaration[] _facet_decl = decl.getContents(false, DeclarationKind.dk_provides);    
    
    if (_facet_decl.length != 0)
    {
      ct.getImpl().addContextValue("facet_list", _facet_decl);
    }
    
    // check for imported interface
    
    ProvidesDecl _prov_decl = null;
    Declaration[] _tmp;
   
    for (int i = 0; i < _facet_decl.length; i++)
    {
      // check the interface declaration
      _prov_decl = (ProvidesDecl) _facet_decl[i];
      // take the first dependency
      _tmp = _prov_decl.getDependencies();
      // search in the ast (normally should be in !!!!!    
      Declaration itf_decl = _ast.lookup(_tmp[0].getAbsoluteName());
      // check if it is an import   
      if(!(itf_decl.getAbsoluteName().startsWith(_scope_name)) && (itf_decl != null))
      {
        mapInterfaceMI((InterfaceDecl) itf_decl, pkg_name);
      }
    }
    
    //////////////////////////
    // receptacle port type MI
    //////////////////////////
    
    // get the primary declaration 
    
    Declaration[] _receptacle_decl = decl.getContents(false,DeclarationKind.dk_uses);
  
    if(_receptacle_decl.length != 0)
    {
      ct.getImpl().addContextValue("receptacle_list", _receptacle_decl);
    }
    
    // check for imported interface
    
    UsesDecl _uses_decl = null;
    
    for (int i = 0; i < _receptacle_decl.length; i++)
    {
      //    check the interface declaration
      _uses_decl = (UsesDecl) _receptacle_decl[i];
      // take the first dependency
      _tmp = _uses_decl.getDependencies();
      // search in the ast (normally should be in !!!!!    
      Declaration itf_decl = _ast.lookup(_tmp[0].getAbsoluteName());
      // check if it is an import   
      if (!(itf_decl.getAbsoluteName().startsWith(_scope_name))
        && (itf_decl != null))
      {
        mapInterfaceMI((InterfaceDecl) itf_decl, pkg_name);
      }
    }
    
    //  consumer port type MI
    
    Declaration[] _consumer_decl = decl.getContents(false,DeclarationKind.dk_consumes);
   
    if(_consumer_decl.length != 0)
    {
      ct.getImpl().addContextValue("consumer_list", _consumer_decl);
    }

    // emitter port type MI
    
    Declaration[] _emitter_decl = decl.getContents(false, DeclarationKind.dk_emits);
   
    if(_emitter_decl.length != 0)
    {
      ct.getImpl().addContextValue("emitter_list", _emitter_decl);
    }
    
    // publisher port type MI
    
    Declaration[] _publisher_decl = decl.getContents(false, DeclarationKind.dk_publishes);
   
    if(_publisher_decl.length != 0)
    {
      ct.getImpl().addContextValue("publisher_list", _publisher_decl);
    }
    
    // constructor is created and ready to by generated
    
    clazz.addConstructor(ct);
    _repository.addObject(clazz);
    
    
    // a component also has an equivalent interface !
    // As a component can offer a receptacle that reference an
    // other component, home or interface.
    // the creation of the home equivalent interface is delegated 
    // to an other class.
    
    // naming the class 
    
    String comp_equv_itf_mi_class_name = decl.getName() + "_interfaceMI"; // name of the class
    clazz = new ClassObjectImpl(comp_equv_itf_mi_class_name);    // create its representation

    // filling the class representation
 
    // some comments on generated class
    clazz.addComment("This class represents the component equivalent interface type MetaInformation ");
    clazz.addComment("for the given " + decl.getName());
    // class is public
    clazz.setModifier(ModifierKindImpl.mk_public);
    // set the package name for home MI class
    clazz.setPackage(pkg_name);           
    // this class inherits from the home type MI implementation         
    clazz.addInheritedObject("org.objectweb.openccm.Containers.MetaInformation.InterfaceTypeImpl");

    clazz.addImport("org.objectweb.openccm.Containers.MetaInformation.*");
    // add specific home declaration elements
    // in the constructor

    ct = new ConstructorObjectImpl();
    ct.getImpl().setMacro("COMP_EQUIV_ITF"); // set velocity macro the given component equivalent interface
    AttributeObject param1 = new AttributeObjectImpl();
    param1.setName("type_ref");
    param1.setType("TypeImpl");
    ct.addParameter(param1);
    AttributeObject param2 = new AttributeObjectImpl();
    param2.setName("type_ki");
    param2.setType("type_kind");
    ct.addParameter(param2);
  
    // Fill the specific Component equivalent interface Type MI
  
    ct.getImpl().addContextValue("name", "???"); //   TODO : find the good name
    ct.getImpl().addContextValue("rep_id", "???"); // TODO : find the good repository ID
    ct.getImpl().addContextValue("type_instance_factory_entrypt",
                                    "???"); // TODO : find the good entrypt

    // add the specific operations to the interface
    
    Declaration[] _equiv_itf_decl = CCMObj_scope.getContents(false, DeclarationKind.dk_operation);
        
    ct.getImpl().addContextValue("opt_list",_equiv_itf_decl);
      
    clazz.addConstructor(ct);

    // the class is finished and ready to be generated

    _repository.addObject(clazz);   
    
  };
  
  /**
   * mapInterfaceMI
   * 
   * generates the appropriate type MI class for a given 
   * interface declaration. Map the componnet type MI 
   * in a class
   * 
   * @param _decl - the interface declaration
   * @param pkg_name   - the interface package name
   * 
   */
  
  private void mapInterfaceMI(InterfaceDecl decl, String pkg_name)
  {
    ClassObject       clazz = null;
    ConstructorObject ct    = null;
    
    // retrieve the name and create the class obj

    String class_name = new String(decl.getName() + "_interfaceMI");
    clazz = new ClassObjectImpl(class_name);

    // filling the class representation
    clazz.addImport("org.objectweb.openccm.Containers.MetaInformation.*");
    clazz.addComment("This class is the interface type MetaInformation");
    clazz.addComment("for the given interface. It gathers static properties");
    clazz.addComment("of CCM concepts managed by the related interface");
    clazz.setModifier(ModifierKindImpl.mk_public); // class is public
    clazz.setPackage(pkg_name);                    // set the package name           
    clazz.addInheritedObject("org.objectweb.openccm.Containers.MetaInformation.InterfaceTypeImpl");

    // add the specific code for managed CCM concept for 
    // a given component, from its IDL declaration

    ct = new ConstructorObjectImpl();
    ct.getImpl().setMacro("INTERFACE_MI_CONSTRUCTOR"); // set velocity macro
    AttributeObject param1 = new AttributeObjectImpl();
    param1.setName("type_ref");
    param1.setType("TypeImpl");
    ct.addParameter(param1);
    AttributeObject param2 = new AttributeObjectImpl();
    param2.setName("type_ki");
    param2.setType("type_kind");
    ct.addParameter(param2);
    
    // specific interface type MetaInformation
    
    ct.getImpl().addContextValue("name", decl.getName());
    ct.getImpl().addContextValue("rep_id",decl.getId());
    ct.getImpl().addContextValue("type_instance_factory_entrypt",
                                 "???"); 
    // managed attributes
    
    Declaration[] att_decl = decl.getContents(true, DeclarationKind.dk_attribute);
    
    if(att_decl.length != 0)
    {
      ct.getImpl().addContextValue("att_list", att_decl);
    }
    
    // managed operation
    
    Declaration[] opt_decl = decl.getContents(true, DeclarationKind.dk_operation);
    
    if(opt_decl.length != 0)
    {
      ct.getImpl().addContextValue("opt_list", opt_decl);
    }
    
    clazz.addConstructor(ct);
    _repository.addObject(clazz);
  };
  

  // ==================================================================
  //
  // Public methods.
  //
  // ==================================================================
  
  /**
   * idl_to_java
   * 
   * Generate java files for the MetaInformation ofType.
   *
   * @param base_dir - The base directory where files will be generated.
   * @param scope - The scope to map.
   *
   */
  
  public void idl_to_java(String outputDir, Scope scope)
  throws org.objectweb.openccm.generator.common.lib.GenerationException
  {
    // Local Variables
    int i;
    String   package_name;
    java.io.File out_dir       = null;
    _repository                = new RepositoryImpl();
 
    Declaration[] declared_homes      = null;
    Declaration[] declared_components = null;
    Declaration[] declared_interfaces = null;

    // get a local reference to the scope 
    _scope_name = scope.getAbsoluteName();
    
    // Check and create the output directory
    try{
        out_dir = FileManager.mkdir(outputDir);
        // TODO : OLD CODE out_dir = _translator.mkdir(outputDir);
    }catch(GenerationException ex){
        System.err.println(ex.getMessage());
        return;
    }
    
    // The directory is now created the treatment for
    // declaration contained in the IDL can begin
    // retrieve all pertinent declaration for type MI
    
    declared_homes      = scope.getContents(false, DeclarationKind.dk_home); // for homes
    declared_components = scope.getContents(false, DeclarationKind.dk_component); // for components
    declared_interfaces = scope.getContents(false, DeclarationKind.dk_interface);//getAllDeclarations(scope, DeclarationKind.dk_local_interface); // for interfaces
    
    // The specific treatment could be performed on each
    // declaration found in the IDL scope
    
    // For each declared home, call the home MI mapping
    
    for(i=0;i<declared_homes.length;i++)
    {
      // retrieve home declaration from the declaration array
      
      HomeDecl _home_decl = (HomeDecl) declared_homes[i]; // retrieve the declaration
      package_name = _translator.getPackage(_home_decl); // retrieve the package_name
      mapHomeMI(_home_decl, package_name);               // call the Home type MI mapping
    }
    
    // for each declared component, call the component MI mapping
    
    for(i=0;i<declared_components.length;i++)
    {
      ComponentDecl _comp_decl = (ComponentDecl) declared_components[i]; // retrieve the declaration
      package_name = _translator.getPackage(_comp_decl); // retrieve the package_name
      mapComponentMI(_comp_decl,package_name); // call the component MI mapping 
    }
    
    // for each declared interface, call the interface MI mapping
    
    for(i=0;i<declared_interfaces.length;i++)
    {
      InterfaceDecl _intf_decls = (InterfaceDecl) declared_interfaces[i];
      package_name = _translator.getPackage(_intf_decls);
      mapInterfaceMI(_intf_decls, package_name);
    }
    
    // mappings are done for type MI, the java files
    // can be generated
       
    org.objectweb.openccm.generator.java.core.lib.JavaGenerator java_gen = new JavaGenerator();
    java.util.Vector v = new java.util.Vector();
    v.add("org/objectweb/corba/generator/metainformation/metainformation.vm");
    java_gen.initialize("metainformation", v);
    java_gen.generate(_repository, outputDir);
  }
}
