/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2003 USTL - LIFL - GOAL
Contact: openccm-team@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Jerome Offroy.
Contributor(s): ______________________________________.

====================================================================*/

package org.objectweb.util.monolog.provider.lib;

import java.io.FileInputStream;
import java.util.Properties;
import java.util.TreeSet;

import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.HandlerFactory;
import org.objectweb.util.monolog.api.Level;
import org.objectweb.util.monolog.api.LevelFactory;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.LoggerFactory;
import org.objectweb.util.monolog.file.monolog.PropertiesConfAccess;
import org.objectweb.util.monolog.provider.api.LoggerProvider;
import org.objectweb.util.monolog.provider.lib.topics.BundleTopics;


/**
 *  This class provides utility method for using Monolog
 *  This class is actually a wrapper on the unique loggerFactory of the current JVM
 * 
 * @author Jerome OFFROY.
 */
public class LoggerProviderImpl implements LoggerProvider {
    /** 
     * default config file name for monolog configuration
     */
    private String configFile = "monolog.CCM.properties";

    /**
     * use or not classpath to find file named configFile
     */
    private boolean usecp = false;

    /**
     * The Singleton Local LoggerFactory
     */
    private LoggerFactory lf = null;

    /**
     * Properties used for Logging system
     */
    private Properties props = null;

    /**
     * constructor
     */
    public LoggerProviderImpl() {
    }

    /**
     * Configure Logger for OpenCCM modules
     * This is called only from the ComponentServer.
     */
    public void configure(String file, boolean useclasspath) {
        configFile = file;
        usecp = useclasspath;
        LoggerFactory lf = getLoggerFactory();             
    }

    /**
     * It returns the unique LoggerFactory used in OpenCCM.
     * initialize it if not already done.
     */
    public synchronized LoggerFactory getLoggerFactory() {
        if (lf == null) {
            try {
                // load properties from config file
                props = new Properties();
                if (usecp) {
                    props.load(ClassLoader.getSystemResourceAsStream(configFile));
                } else {
                    props.load(new FileInputStream(configFile));
                }
                // Instanciate the LoggerFactory
                String b = props.getProperty("log.config.classname", null);
                if (b == null) {
                    System.err.println("Malformed configuration log file: log.config.classname not available");
                    return null;
                }
                lf = (LoggerFactory) Class.forName(b).newInstance();
                // Configure the LoggerFactory with the properties
                PropertiesConfAccess.load(props, lf, (HandlerFactory) lf, (LevelFactory) lf);
            } catch (Exception e) {
                System.err.println("Cannot get LoggerFactory:" + e);                
                e.printStackTrace();
            }
        }
        return lf;
    }

    /**
     * Shortcut that returns the LevelFactory
     */
    public LevelFactory getLevelFactory() {
        return (LevelFactory) getLoggerFactory();
    }

    /**
     * Shortcut to get the Logger by its topic name.
     * @param topic is the topic of the returned logger
     * @return always a logger instance (never null value).
     */
    public Logger getLogger(String topic) {
        return getLoggerFactory().getLogger(topic);
    }

    /**
     * @see org.objectweb.util.monolog.provider.api.LoggerProvider#getLogger(String, Object)
     */
    public Logger getLogger(String topic, Class caller) {
        String preTopic = TopicProviderSingleton.getTopicProvider().getTopic(caller);
        getLogger(BundleTopics.getPreTopic("PRETOPIC_OPENCCM_DEFAULT")).log(BasicLevel.DEBUG,
            "The Topic of " + caller + " is : " + preTopic);
        return getLoggerFactory().getLogger(preTopic);
    }

    /**
     * @see org.objectweb.util.monolog.provider.api.LoggerProvider#createLogger(String, Object, String[])
     */
    public Logger createLogger(String topic, Class caller, String[] description) {
        TopicProviderSingleton.createTopic(topic, caller, description);
        return getLogger(topic, caller);
    }


    /**
     * get Topics defined in Monolog. 
     * Assumes that all Loggers are TopicalLoggers.
     */
    public String[] getTopics() {
        Logger[] logs = getLoggerFactory().getLoggers();
        // put names in alphabetical order
        TreeSet tset = new TreeSet();
        for (int i = 0; i < logs.length; i++) {
            tset.add(logs[i].getName());
        }
        return (String[]) tset.toArray(new String[0]);
    }

    /**
     * get Topic Level
     */
    public String getTopicLevel(String topic) {
        String ret = null;
        Logger log = getLogger(topic);
        Level lev = log.getCurrentLevel();
        return lev.getName();
    }

    /**
     * set Topic Level
     */
    public void setTopicLevel(String topic, String l) {
        Logger log = getLogger(topic);
        Level lev = getLevelFactory().getLevel(l);
        // must check null (bug monolog)
        if (lev != null) {
            log.setLevel(lev);
        } else {
           
        }
    }

    /**
     * returns list of properties for logging system
     */
    public Properties getProperties() {
        if (props == null) {
            getLoggerFactory();
        }
        return props;
    }

}
