/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA - USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle, Mathieu Vadet.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.ast.lib;

/** To access AST DeclarationKind. */
import org.objectweb.openccm.ast.api.DeclarationKind;

/** To access AST Declaration. */
import org.objectweb.openccm.ast.api.Declaration;

/**
 * ModuleDeclImpl is a wrapper class for IDL module declarations.
 *
 * Inherits from:
 * - Scope as IDL modules are IDL scopes.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.3
 */

public class ModuleDeclImpl
     extends ForwardScopeIDLImpl
  implements org.objectweb.openccm.ast.api.ModuleDecl,
             org.objectweb.openccm.ast.api.PsdlModuleDecl,
             org.objectweb.openccm.ast.api.CidlModuleDecl
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Reference to the CORBA 3.0 ModuleDef. */
    private org.omg.CORBA.ComponentIR.ModuleDef component_module_def_;

    /**
     * TODO: This variable is already defined in ForwardScopeImpl!
     */
    private boolean completed_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /**
     * The constructor with the parent scope.
     *
     * @param rep The repository of the declaration.
     * @param parent The parent scope of the module declaration.
     */
    protected
    ModuleDeclImpl(Repository rep,
                   ScopeImpl parent)
    {
        // Call the ForwardScopeIDLImpl constructor.
        super(rep, parent);

        // Init internal state.
        component_module_def_ = null;
        completed_ = false;

    //    System.out.println("ModuleDeclImpl::ModuleDeclImpl " + this + " in " + parent);
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for DeclarationImpl.
    //
    // ==================================================================

    /**
     * Loads infos of the CORBA 3.0 ModuleDef.
     *
     * @param moduleDef The ModuleDef to load.
     */
    protected void
    load(org.omg.CORBA.Contained contained)
    {
        component_module_def_ = org.omg.CORBA.ComponentIR.ModuleDefHelper.
                                narrow(contained);
        super.load(contained);
    }

    /**
     * Loads infos of the CORBA 3.0 ModuleDef.
     *
     * @param moduleDef The ModuleDef to load.
     */
    protected void
    loadAsMapping(org.omg.CORBA.Contained contained)
    {
        component_module_def_ = org.omg.CORBA.ComponentIR.ModuleDefHelper.
                                narrow(contained);
        super.loadAsMapping(contained);
    }

    /**
     * Obtain its CORBA 3.0 Contained reference.
     *
     * @return The Contained object associated with the module declaration.
     */
    protected org.omg.CORBA.Contained
    getContained()
    {
       return component_module_def_;
    }

    // ==================================================================
    //
    // Internal methods for ScopeImpl.
    //
    // ==================================================================

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the module declaration.
     **/
    protected org.omg.CORBA.Container
    getContainer()
    {
       return component_module_def_;
    }

    /**
     **  Obtain its CORBA 3.0 Container reference.
     **
     **  @return The Container object associated with the module declaration.
     **/
    protected org.omg.CORBA.ComponentIR.Container
    getComponentContainer()
    {
       return component_module_def_;
    }

    // ==================================================================
    //
    // Internal methods for ForwardScopeImpl.
    //
    // ==================================================================

    // ==================================================================
    //
    // Internal methods for ForwardScopeIDLImpl.
    //
    // ==================================================================

    /**
     * Create the container object.
     */
    protected void
    createContainer()
    {
        // Create it only if it was not previously created.
        if (component_module_def_ == null)
        {
           // Create a ModuleDef into the IFR.
           component_module_def_ =
               the_parent_.getComponentContainer().
               create_component_module(getId(), getName(), getVersion());
        }
    }

    /**
     * Complete the container object.
     */
    protected void
    completeContainer()
    {
        completed_ = true;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.WithDependencies
    //
    // ==================================================================

    /**
     * Obtain the declaration external dependencies.
     *
     * Note: for scopes, contained objects are not considered
     * as dependencies.
     *
     * @return The list of dependencies as an array of Declaration.
     */
    public Declaration[]
    getDependencies()
    {
        if (dependencies_!=null)
            return dependencies_;

        Declaration[] decls = getContents(true,
             org.objectweb.openccm.ast.api.DeclarationKind.dk_all);
        Declaration[] depend = null;
        java.util.List module_depend = new java.util.ArrayList();

        // contents
        for (int i=0; i<decls.length; i++)
        {
            depend = decls[i].getDependencies();
            for (int j=0; j<depend.length; j++)
            {
                if ((!containsDecl(depend[j])) &&
                    (module_depend.indexOf(depend[j])==-1))
                    module_depend.add(depend[j]);
            }
        }

        dependencies_ = (Declaration[])module_depend.toArray(
                                          new Declaration[0]);
        return dependencies_;
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.Declaration
    //
    // ==================================================================

    /**
     * Obtain its DeclarationKind.
     *
     * @return The DeclarationKind of the object.
     */
    public long
    getDeclKind()
    {
        return DeclarationKind.dk_module;
    }

    /**
     * Create the interface declaration into the IFR.
     */
    public void
    create()
    {
        if (getContainer() == null)
        {
            createContainer();
            if (the_parent_!=null)
                the_parent_.addDecl(this);
        }
        else if (!completed_)
            completeContainer();
    }

//
// Added to correct the Bug #300511
// Contributor: Philippe Merle - Pawel Rzepa
//
    /**
     * Destroy all the contained declarations of the scope in the IR3.
     */
    public void
    destroy()
    {
        // Destroy the declarations locally present in the AST.
        for (int i=contained_decls_.size()-1; i>=0; i--)
        {
            Declaration decl = (Declaration)contained_decls_.get(i);
            decl.destroy();
        }

        // Obtain the content of the module from the Interface Repository.
        org.omg.CORBA.Contained[] contents =
            getContainer().contents(org.omg.CORBA.DefinitionKind.dk_all, true);

        // Destroy the module if it is empty in the Interface Repository.
        if (contents.length==0)
        {
            super.destroy();
        }
    }

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.Scope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.ForwardScope
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.ModuleDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.PsdlModuleDecl
    //
    // ==================================================================

    // ==================================================================
    //
    // Methods for OMG IDL org.objectweb.openccm.ast.api.CidlModuleDecl
    //
    // ==================================================================
}
