/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.generator.java.core.lib;

// Package dependencies
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.translator.idl2java.api.IDL_JavaTranslator;
import org.objectweb.openccm.generator.common.lib.GenerationException;
import java.util.ArrayList;
import java.util.List;
import java.io.File;


/**
 * This class is a java generator. Generation is done from a java AST.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</A>
 */

public class JavaGenerator
     extends org.objectweb.openccm.generator.common.lib.Generator
  implements org.objectweb.openccm.generator.java.core.api.JavaGenerator
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The output base directory */
    private File base_dir_;

    /** Utility class to convert types in Java*/
    public IDL_JavaTranslator translator_;

    // ==================================================================
    //
    // Constructor.
    //
    // ==================================================================

    /** The default constructor. */
    public
    JavaGenerator()
    {
        super();
        disableLog();
        // Init internal state
        translator_ = new org.objectweb.openccm.generator.translator.idl2java.lib.IDL_JavaTranslator();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Generate a java class.
     *
     * @param rep - The repository to generate.
     **/
    private void
    map_class(ClassObject clazz)
    {
        // Map header
        create_file(clazz, "out");
        put("package_name", clazz.getPackage());
        map("FILE_HEADER");

        // Map class
        put("modifier", compute_class_prefix(clazz));
        put("class_imports", clazz.getImports());
        put("class_comments", clazz.getComments());
        put("class_name", clazz.getName());
        put("extends_list", clazz.getInheritedObjects());
        put("impl_list", clazz.getImplementedObjects());
        put("att_list", clazz.getAttributes());
        put("constructor_list", clazz.getConstructors());
        put("method_list", clazz.getMethods());
        map("CLASS");

        // End of mapping
        close("out");
    }

    /**
     * Generate a java interface.
     *
     * @param rep - The repository to generate.
     **/
    private void
    map_interface(InterfaceObject itf)
    {
        // Map header
        create_file(itf, "out");
        put("package_name", itf.getPackage());
        map("FILE_HEADER");

        // Map interface
        put("modifier", compute_class_prefix(itf));
        put("itf_imports", itf.getImports());
        put("itf_comments", itf.getComments());
        put("itf_name", itf.getName());
        put("extends_list", itf.getInheritedObjects());
        put("itf_method_list", itf.getMethods());
        map("INTERFACE");

        // End of mapping
        close("out");
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Initialize the generator.
     *
     * @param app_name - The application's name.
     * @param templates - Templates files to use.
     **/
    public void
    initialize( String app_name,
                List templates )
    {
        List list = new java.util.ArrayList();

        /** Initialize environment **/

        // Templates to use.
        list.clear();
        list.add("org/objectweb/openccm/generator/common/common.vm");
        list.add("org/objectweb/openccm/generator/java/core/java.vm");
        list.addAll(templates);
        setLibrary(list);

        // call the common generator init method
        super.init();

        /** Initialize velocity context **/
        put("which", app_name);
        put("gen", this);
    }

    /**
     * Generate java files from the repository.
     *
     * @param base_dir - The base directory where files will be generated.
     **/
    public void
    generate(Repository rep, String base_dir)
    {
        ArrayList decls = null;

        // Check and/or create the output base dir
        // Check and create the output directory
        try{
            base_dir_ = org.objectweb.openccm.generator.common.lib.FileManager.mkdir(base_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }

        // Get Repository contents
        decls = rep.getContents();

        for (int i=0; i<decls.size(); i++)
        {
            // Map a class
            try{
                ClassObject clazz = (ClassObject)decls.get(i);
                map_class(clazz);
            }catch(ClassCastException ex1){
                // Map an interface
                try{
                    InterfaceObject itf = (InterfaceObject)decls.get(i);
                    map_interface(itf);
                }catch(ClassCastException ex2){
                    // Should not happen
                    ex2.printStackTrace();
                }
            }
        }
    }

    /**
     * Create the output file.
     * This include creation of package directories if necessary.
     *
     * @param object - The class or interface to generate.
     * @param id - An identifiant for this stream.
     **/
    public void
    create_file(InterfaceObject object, String id)
    {
        // Create the target directory
        String filename = null;
        File target_dir = null;

        target_dir = new File( base_dir_.getAbsolutePath() +
                               File.separatorChar +
                               translator_.getAsDirectory(object.getPackage()) );
        try{
            org.objectweb.openccm.generator.common.lib.FileManager.mkdir(target_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }

        filename = new String( target_dir.getAbsolutePath() +
                               File.separatorChar +
                               object.getName() + ".java" );
        System.out.println( "File '" + filename + "' has been generated");

        // create the target file and map
        open(filename, id);
    }

    // ==================================================================
    //
    // Useful methods for java mapping.
    //
    // ==================================================================

    public String
    compute_class_prefix(InterfaceObject obj)
    {
        String result = "";

        result = obj.getModifier().toString();
        if (obj.isAbstract())
            result += " abstract";
        else if(obj.isFinal())
            result += " final";

        return result;
    }
}
