/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Philippe Merle.
Contributor(s): Mathieu Vadet, Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.ir3;

// Package dependencies
import org.omg.CORBA.*;

/** 
 * Implementation of the CORBA::OperationDef interface.
 *
 * @author <a href="mailto:Philippe.Merle@lifl.fr">Philippe Merle</a>
 *         <a href="mailto:Mathieu.Vadet@lifl.fr">Mathieu Vadet</a>
 *
 * @version 0.5
 */

public class OperationDef_impl
     extends FeatureContained
  implements OperationDefOperations
{
    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The result type of the operation. */
    protected IDLType_ref result_def_;

    /** The parameters of the operation. */
    protected Parameters params_;

    /** The mode of the operation. */
    protected OperationMode mode_;

    /** The contexts of the operation. */
    protected String[] contexts_;

    /** The exceptions of the operation. */
    protected ExceptionDef_ref[] exceptions_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /** TODO. */
    public
    OperationDef_impl(IFR ifr,
                      Container_impl container)
    {
        this(ifr, container, false);

        // Sets the tie servant object.
        setServant(new OperationDefPOATie(this));
    }

    /** TODO. */
    public
    OperationDef_impl(IFR ifr,
                      Container_impl container,
                      boolean dummy)
    {
        // Calls the FeatureContained contructor.
        super(ifr, container);

        // Creates the result_def reference.
        result_def_ = new IDLType_ref(this,
                              ifr.getRepository().getPrimDefVoid());

        // Inits internal state.
        params_ = new Parameters(this);
        mode_ = OperationMode.OP_NORMAL;
        contexts_ = null;
        exceptions_ = null;
    }

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Obtain its CORBA::OperationDef object reference.
     */
    public OperationDef
    asOperationDef()
    {
        return OperationDefHelper.narrow(asObject());
    }

    /** 
     * Obtain its CORBA::OperationDescription.
     */
    public OperationDescription
    getOperationDescription()
    {
        // create the CORBA::OperationDescription.
        OperationDescription description = new OperationDescription();

        // set its fields.
        description.name = name();
        description.id = id();
        description.defined_in = getContainerID();
        description.version = version();
        description.result = result();
        description.mode = mode();
        description.contexts = contexts();
        description.parameters = params();
        description.exceptions =
            ExceptionDef_impl.compute_ExcDescriptionSeq(exceptions_);

        return description;
    }

    // ==================================================================
    //
    // For Contained_impl class.
    //
    // ==================================================================

    /** 
     * To set the value field of the CORBA::Contained::Description struct.
     */
    public void
    setDescriptionValue(Any any)
    {
        // insert its CORBA::OperationDescription into the any.
        OperationDescriptionHelper.insert(any, getOperationDescription());
    }

    // ==================================================================
    //
    // Internal methods for managing dependencies.
    //
    // ==================================================================

    /** 
     * Cuts dependencies to other objects.
     */
    protected void
    cutDependencies()
    {
        // Cuts the dependency.
        result_def_.cutDependency();

        // Cuts dependencies to params types.
        params_.cutDependencies();

        mode_ = null;
        contexts_ = null;

        // Cuts the dependency to exceptions.
        ExceptionDef_ref.cutDependencies(exceptions_);
        exceptions_ = null;

        // Calls the Container_impl::cutDependencies operation.
        super.cutDependencies();
    }

    // ==================================================================
    //
    // For CORBA::IRObject interface.
    //
    // ==================================================================

    /** 
     * IDL:omg.org/CORBA/IRObject/def_kind:1.0
     */
    public DefinitionKind
    def_kind()
    {
        return DefinitionKind.dk_Operation;
    }

    // ==================================================================
    //
    // For CORBA::OperationDef interface.
    //
    // ==================================================================

    /** 
     * IDL:omg.org/CORBA/OperationDef/result:1.0
     */
    public TypeCode
    result()
    {
        // delegate to the IDLType reference.
        return result_def_.recursiveType();
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/result_def:1.0
     */
    public IDLType
    result_def()
    {
        // delegate to the IDLType reference.
        return result_def_.asIDLType();
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/result_def:1.0
     */
    public void
    result_def(IDLType val)
    {
        // try to cast val to a local object
        // and delegate to the IDLType reference.
        result_def_.setDependency( castToLocal(val) );
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/params:1.0
     */
    public ParameterDescription[]
    params()
    {
        // delegate to the Parameters.
        return params_.getParameterDescriptionSeq();
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/params:1.0
     */
    public void
    params(ParameterDescription[] val)
    {
        // Check if an oneway operation that parameters are only PARAM_IN!
        if(mode_ == OperationMode.OP_ONEWAY)
            for(int i=0; i<val.length; i++)
                if(val[i].mode != ParameterMode.PARAM_IN)
                    throw exceptionNeedOnlyInParams();

        // delegate to the Parameters.
        params_.setParameterDescriptionSeq(val);
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/mode:1.0
     */
    public OperationMode
    mode()
    {
        return mode_;
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/mode:1.0
     */
    public void
    mode(OperationMode val)
    {
        // Check if ONEWAY only PARAM_IN and no exceptions!
        if(val == OperationMode.OP_ONEWAY)
    {
            params_.checkOnlyInModes();
            if(exceptions_ != null && exceptions_.length != 0)
               throw exceptionNoExceptionsForOnewayOperation();
        }
        mode_ = val;
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/contexts:1.0
     */
    public String[]
    contexts()
    {
        return contexts_;
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/contexts:1.0
     */
    public void
    contexts(String[] val)
    {
        // TODO: check unicity
        contexts_ = val;
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/exceptions:1.0
     */
    public ExceptionDef[]
    exceptions()
    {
        ExceptionDef[] result = new ExceptionDef[exceptions_.length];

        for(int i=0; i<exceptions_.length; i++)
             result[i] = exceptions_[i].asExceptionDef();

        return result;
    }

    /** 
     * IDL:omg.org/CORBA/OperationDef/exceptions:1.0
     */
    public void
    exceptions(ExceptionDef[] val)
    {
        // Check if ONEWAY then no exceptions!
        if(mode_ == OperationMode.OP_ONEWAY)
    {
            if(val.length != 0)
               throw exceptionNoExceptionsForOnewayOperation();
        }

        ExceptionDef_impl[] exceptions = new ExceptionDef_impl[val.length];
        for (int i=0; i<val.length; i++)
            exceptions[i] = castToLocal(val[i]);

        // TODO: check unicity

        ExceptionDef_ref[] refs = new ExceptionDef_ref[val.length];
        for (int i=0; i<val.length; i++)
            refs[i] = new ExceptionDef_ref( this, exceptions[i] );

        // cut the previous dependency to exceptions.
        ExceptionDef_ref.cutDependencies(exceptions_);

        exceptions_ = refs;
    }

    // ==================================================================
    //
    // For IDL2 mapping.
    //
    // ==================================================================

    /** 
     * TODO 
     */
    protected void
    whenCreated()
    {
        // do nothing if mappings are not enabled.
        if (!getIFR().getRepository().enable_mappings_)
            return ;

        // check the container kind
        if (container_.def_kind().value()!=org.omg.CORBA.DefinitionKind._dk_Home)
            return ;

        // else create the mapping according to the CCM spec

        //
        //  IDL2 mapping
        //
        //
        //  copy the operation in the home explicit interface
        //
        InterfaceDef_impl explicit = ((HomeDef_impl)container_).base_interfaces_[0].getImpl();
        String explicit_base_id = explicit.getBaseID();
        String id = explicit_base_id + name() + ":" + version();

        container_.mapping_started_ = true;
        explicit.create_operation(id, name(), version(), result_def(), mode(),
                                  params(), exceptions(), contexts());
        container_.mapping_started_ = false;

        //
        // CIDL mapping
        //
        //
        //  copy the operation in the home explicit executor interface
        //
        String cont_base_id = container_.getBaseContainerID();

        String explicit_exe_id = cont_base_id + "CCM_" + container_.name() + "Explicit:" + container_.version();
        org.omg.CORBA.ExtLocalInterfaceDef explicit_exe = org.omg.CORBA.ExtLocalInterfaceDefHelper.narrow(
             getIFR().getRepository().lookup_mapped_id(explicit_exe_id));

        id = cont_base_id + "CCM_" + container_.name() + "Explicit/" + name() + ":" + container_.version();
        explicit_exe.create_operation(id, name(), version(),
                                      result_def(),
                                      mode(),
                                      params(),
                                      exceptions(),
                                      contexts());
    }
}
