/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Christophe Demarey.
Contributor(s): ________________________________________.

====================================================================*/

package org.objectweb.openccm.pss.generator.common.lib;

// Package dependencies
/** To access AST. */
import org.objectweb.openccm.ast.api.StorageTypeStateMemberDecl;
import org.objectweb.openccm.ast.api.AbstractStorageTypeDecl;
/** To access Java AST. */
import org.objectweb.openccm.generator.java.ast.api.*;
import org.objectweb.openccm.generator.java.ast.lib.*;
/** Others. */
import org.objectweb.openccm.generator.common.lib.GenerationException;
import org.objectweb.openccm.generator.translator.idl2java.api.PSDL_JavaTranslator;
import org.objectweb.openccm.pss.generator.common.api.PSDL2JavaGenerator;


/**
 * This class generates the mapping for a State Member.
 *
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 * 
 * @version 0.1
 */
public class StateMemberMapping
  implements org.objectweb.openccm.pss.generator.common.api.StateMemberMapping
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** The State member declaration. */
    protected StorageTypeStateMemberDecl state_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public StateMemberMapping()
    {
        // Init internal state
        state_ = null;
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /** 
     * Set the State Memeber to map. 
     *
     * @param state - The State Member to map.
     */
     public void
     setStateMember(StorageTypeStateMemberDecl state)
     {
         state_ = state;
     }

    /**
     * Map the name of the state member.
     * 
     * @return The name of the mapping attribute.
     */
    public String
    mapStateName()
    {
        return mapStateName( state_.getName() );
    }

    /**
     * Get the name of the mapping attribute.
     * 
     * @return The state name.
     */
    protected String
    mapMarshalledStateName()
    {
        return mapMarshalledStateName( state_.getName() );
    }

    /**
     * Map a Storage Type State Member.
     *
     * @param translator - A Java utility class to translate types.
     * @param obj - The StorageType mapping object.
     * @param finalAccessor - True if accessors must be final.
     */
    public void
    toJavaAccessors( PSDL_JavaTranslator translator,
                     InterfaceObject obj,
                     boolean finalAccessor )
    {
        // Must be implemented in sub-classes.
    }

    /**
     * Map a Read Only Storage Type State Member to a user non-available modifier.
     *
     * @param translator - A Java utility class to translate types.
     * @param clazz - The StorageType mapping object.
     */
    public void
    toJavaReadOnlyStateMemberModifier( PSDL_JavaTranslator translator,
                                       ClassObject clazz )
    {
        MethodObject method = null;
        ParameterObject param = null;

        method = new MethodObjectImpl();
        method.addComment("Set the "+state_.getName()+" value.");
        method.addComment(" ");
        method.addComment("@param initial_value - The value to set.");
        method.setName( "set_" + state_.getName() );
        method.setReturnType("void");
        method.setFinal(true);
        param = new ParameterObjectImpl();
        param.setName("initial_value");
        param.setType( translator.toJava(state_.getType()) );
        method.addParameter(param);
        method.getImpl().setMacro("SET_METHOD");
        method.getImpl().addContextValue("var", mapStateName());
        method.getImpl().addContextValue("value", "initial_value");
        clazz.addMethod(method);
    }

    /**
     * Map a Storage Type State Member to an attribute.
     *
     * @param translator - A Java utility class to translate types.
     * @param clazz - The StorageType mapping object.
     * @param st - The StorageType declaration.
     */
    public void
    toJavaAttribute(PSDL_JavaTranslator translator,
                    ClassObject clazz,
                    org.objectweb.openccm.ast.api.StorageTypeDecl st)
    {
        AttributeObject att = null;
        String marshalled_type = null;

        att = new AttributeObjectImpl();
        att.addComment("The "+state_.getName()+" state.");
        att.setName( mapStateName() );
        att.setType( translator.toJava(state_.getType()) );
        att.setModifier( ModifierKindImpl.mk_private );
        clazz.addAttribute(att);

        // Add the mapping attribute if necessary
        marshalled_type = PSDLMappingTools.getMarshalledType(st, state_);
        if (marshalled_type != null)
        {
            // Add a XXX_marshalled attribute
            att = new AttributeObjectImpl();
            att.addComment("The " + state_.getName() + " marshalled state.");
            att.setName( mapMarshalledStateName() );
            att.setType(marshalled_type);
            att.setModifier( ModifierKindImpl.mk_private );
            clazz.addAttribute(att);
        }
    }

    /**
     * Map a state member to a java parameter.
     *
     * @param translator - A Java utility class to translate types.
     * @param method - The method that will contain parameters.
     **/
    public void
    toJavaParameter( PSDL_JavaTranslator translator,
                     MethodObject method )
    {
        ParameterObject param = null;

        // Create the corresponding parameter
        param = new ParameterObjectImpl();
        param.setName( state_.getName() );
        param.setType( translator.toJava(state_.getType()) );
        method.addParameter(param);
    }

    /**
     * Map a state list (string[]) to ParameterObject for the java AST.
     *
     * @param generator - The PSDL to Java generator.
     * @param states - The state list.
     * @param method - The method that will contain parameters.
     * @param ast - The storage type containing states.
     **/
    public static void
    mapStateListToParameters( PSDL2JavaGenerator generator,
                              String[] states,
                              MethodObject method,
                              AbstractStorageTypeDecl ast )
    throws GenerationException
    {
        StorageTypeStateMemberDecl state = null;
        ParameterObject param = null;

        for (int i=0; i<states.length; i++)
        {
            // Find the state in the current storageType
            state = (StorageTypeStateMemberDecl)generator.getDeclaration(ast, states[i]);

            if (state == null)
            {
                AbstractStorageTypeDecl[] implemented = null;

                // Find in the implemented classes
                implemented = ast.getAllImplementedAbstractStorageTypes().getAbstractStorageTypes();
                if (implemented.length > 0)
                {
                    int j = 0;
                    while( (j<implemented.length) && (state == null) )
                    {
                        state = (StorageTypeStateMemberDecl)generator.getDeclaration(implemented[j], states[i]);
                        j++;
                    }
                }
                if (state == null)
                {
                    String msg = "State Member '"+states[i]+"' not found!";
                    throw new GenerationException(msg);
                }
            }

            // Create the corresponding parameter
            org.objectweb.openccm.pss.generator.common.api.StateMemberMapping smm
                = new StateMemberMapping();
            smm.setStateMember(state);
            smm.toJavaParameter(generator.getTranslator(), method);
        }
    }

    /**
     * Map the name of the state member.
     * 
     * @param name - The state member name.
     * 
     * @return The name of the mapping attribute.
     */
    public static String
    mapStateName(String name)
    {
        return name + "_state";
    }

    /**
     * Get the name of the mapping attribute.
     * 
     * @param name - The state member name.
     * 
     * @return The state name.
     */
    public static String
    mapMarshalledStateName(String name)
    {
        return name + "_marshalled";
    }
}
