/*====================================================================

OpenCCM: The Open CORBA Component Model Platform
Copyright (C) 2000-2004 INRIA & USTL - LIFL - GOAL
Contact: openccm@objectweb.org

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
USA

Initial developer(s): Alex Andrushchak.
Contributor(s): Christophe Demarey.

====================================================================*/

package org.objectweb.openccm.pss.generator.hibernate.lib;

// Package dependencies
/** To access AST. */
import org.objectweb.openccm.ast.api.StorageTypeDecl;
import org.objectweb.openccm.ast.api.StorageTypeStateMemberDecl;
/** Others. */
import org.objectweb.openccm.pss.generator.common.api.StorageTypeMapping;
import org.objectweb.openccm.pss.generator.common.api.PSDL2JavaGenerator;
import org.objectweb.openccm.pss.generator.common.lib.StateMemberMapping;
import org.objectweb.openccm.generator.common.api.GeneratorBase;
import org.objectweb.openccm.generator.common.lib.GenerationException;
import org.objectweb.openccm.generator.translator.idl2java.api.PSDL_JavaTranslator;
import java.io.File;
import java.util.Enumeration;
import java.util.List;
import java.util.Set;


/**
 * This class generates xml Hibernate files for storage types.
 *
 * @author <a href="mailto:pretend@ukr.net">Alex Andrushchak</a>
 * @author <a href="mailto:Christophe.Demarey@lifl.fr">Christophe Demarey</a>
 */

public class HibernateXmlGenerator
     extends org.objectweb.openccm.generator.common.lib.Generator
  implements org.objectweb.openccm.pss.generator.hibernate.api.HibernateXmlGenerator
{

    // ==================================================================
    //
    // Internal state.
    //
    // ==================================================================

    /** Utility class to convert types in Java */
    public PSDL_JavaTranslator translator_;

    /** A generator to get declarations */
    public GeneratorBase generator_;

    // ==================================================================
    //
    // Constructors.
    //
    // ==================================================================

    /**
     * The default constructor.
     */
    public HibernateXmlGenerator(PSDL2JavaGenerator gen)
    {
        // Init internal state
        translator_ = new org.objectweb.openccm.generator.translator.idl2java.lib.PSDL_JavaTranslator();
        generator_ = gen;
        // No log
        disableLog();
    }

    // ==================================================================
    //
    // Internal methods.
    //
    // ==================================================================

    /**
     * Create the output file.
     * This include creation of package directories if necessary.
     *
     * @param base_dir_str - The base directory where files will be generated.
     * @param package_name - The Storage Type package name.
     * @param st_name - The Storage Type class name.
     * @param id - An identifiant for this stream.
     **/
    private void
    create_file( String base_dir_str,
                 String package_name,
                 String st_name,
                 String id )
    {
        // Create the target directory
        String filename = null;
        File base_dir = null,
             target_dir = null;

        base_dir = new File(base_dir_str);
        target_dir = new File( base_dir.getAbsolutePath() +
                               File.separatorChar +
                               translator_.getAsDirectory(package_name) );
        try{
            org.objectweb.openccm.generator.common.lib.FileManager.mkdir(target_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }

        filename = new String( target_dir.getAbsolutePath() +
                               File.separatorChar +
                               st_name + ".hbm.xml" );
        System.out.println( "File '" + filename + "' has been generated");

        // create the target file and map
        open(filename, id);
    }
    
    /**
     * Get User sub-classes of the Storage Type ST.
     * They will be included in mapping files.
     *
     * @param props - User properties.
     * @param st - A Storage Type.
     */
    private Set
    get_user_subclasses( java.util.Properties props,
                         org.objectweb.openccm.ast.api.StorageTypeDecl st )
    {
        Set user_classes = new java.util.HashSet();
        String name = translator_.getPackage(st) + "." + st.getName();

        if ( props.containsValue(name) )
        {
            String key = null;
            
            for(Enumeration keys = props.keys(); keys.hasMoreElements(); )
            {
                key = (String) keys.nextElement();
                if (props.getProperty(key).compareTo(name) == 0)
                    user_classes.add(key);
            }
        }
        return user_classes;
    }

    /**
     * Has this Storage Type user sub-classes?
     *
     * @param props - User properties.
     * @param st - A Storage Type.
     * 
     * @return True if st has user sub-classes, else false.
     */
    private boolean
    has_user_subclasses( java.util.Properties props,
                         org.objectweb.openccm.ast.api.StorageTypeDecl st )
    {
        return props.containsValue( translator_.getPackage(st) + "." + st.getName() );
    }
    
    /**
     * Get the package of a declaration.
     * 
     * @param name - The declaration name.
     * 
     * @return The declaration's package.
     */
    private String
    getPackage(String name)
    {
        return name.substring(0, name.lastIndexOf("."));
    }
    
    /**
     * Get the short name of a declaration.
     * 
     * @param name - The full declaration name.
     * 
     * @return The declaration's short name.
     */
    private String
    getShortName(String name)
    {
        return name.substring(name.lastIndexOf(".") + 1);
    }
    
    // ==================================================================
    //
    // Public methods.
    //
    // ==================================================================

    /**
     * Initialize the generator.
     *
     * @param app_name - The application's name.
     **/
    public void
    initialize( String app_name )
    {
        List list = new java.util.ArrayList();

        /** Initialize environment **/

        // Templates to use.
        list.clear();
        list.add("org/objectweb/openccm/pss/generator/common/common.vm");
        list.add("org/objectweb/openccm/pss/generator/hibernate/xml_hibernate.vm");
        setLibrary(list);

        // call the common generator init method
        super.init();

        /** Initialize velocity context **/
        put("which", app_name);
    }

    /**
     * Map a storage type to a Hibernate persistent capable classe in the xml file.
     *
     * @param st - The storage type to map.
     **/
    public void
    mapStorageType( org.objectweb.openccm.ast.api.StorageTypeDecl st )
    {
        java.util.List members = null;
        StorageTypeMapping st_mapping = null;
        StorageTypeStateMemberDecl[] sm_array = null;
        String name = null,
               type = null;

        st_mapping = new org.objectweb.openccm.pss.generator.hibernate.lib.StorageTypeMapping();
        st_mapping.setStorageType(st);

        // Map native state members
        members = st_mapping.getStatesToNative(generator_);
        sm_array = (StorageTypeStateMemberDecl[])members.toArray(new StorageTypeStateMemberDecl[0]);
        for (int i=0; i< members.size(); i++)
        {
            put("field_name", StateMemberMapping.mapStateName(sm_array[i].getName()));
            type = translator_.toJava( sm_array[i].getType() );
            put("field_type", type.toLowerCase());
            if (type.compareTo("byte[]") == 0) {
                map("HIBERNATE_FIELD_BLOB");
            } else {
                map("HIBERNATE_FIELD");
            }
        }

        // Map state members to IOR
        members = st_mapping.getStatesToIOR(generator_);
        sm_array = (StorageTypeStateMemberDecl[])members.toArray(new StorageTypeStateMemberDecl[0]);
        for (int i=0; i< members.size(); i++)
        {
            name = StateMemberMapping.mapMarshalledStateName( sm_array[i].getName() );
            put("field_name", name);
            map("HIBERNATE_FIELD_CLOB");
        }

        // Map state members to byte[]
        members = st_mapping.getStatesToCodec(generator_);
        sm_array = (StorageTypeStateMemberDecl[])members.toArray(new StorageTypeStateMemberDecl[0]);
        for (int i=0; i< members.size(); i++)
        {
            name = StateMemberMapping.mapMarshalledStateName( sm_array[i].getName() );
            put("field_name", name);
            map("HIBERNATE_FIELD_BLOB");
        }

        // Map the SH RID state
/*
        put("field_name", "sh_rid");
        // If SH RID can't be fed in a varchar type
        //map("HIBERNATE_FIELD_CLOB");
        map("HIBERNATE_FIELD");
*/
        // Map state member to Class
        members = st_mapping.getStatesToClass(generator_);
        sm_array = (StorageTypeStateMemberDecl[])members.toArray(new StorageTypeStateMemberDecl[0]);
        for (int i=0; i< members.size(); i++)
        {
            name = StateMemberMapping.mapMarshalledStateName( sm_array[i].getName() );
            put("field_name", name);
            map("HIBERNATE_FIELD");
        }
    }

    /**
     * Generate xml Hibernate files.
     *
     * @param base_dir - The base directory where files will be generated.
     * @param storage_types - Storage Types to include in xml Hibernate files.
     **/
    public void
    to_hibernate_xml( String base_dir,
                      List storage_types,
                      String user_properties_file )
    throws GenerationException
    {
        java.io.File out_dir = null;
        java.util.Properties user_props = null;
        java.util.Iterator it = null;
        StorageTypeDecl st = null;

        // Check and create the output directory
        try{
            out_dir = org.objectweb.openccm.generator.common.lib.FileManager.mkdir(base_dir);
        }catch(GenerationException ex){
            System.err.println(ex.getMessage());
            return;
        }

        // Retrieve user properties
        try {
            java.io.InputStream propStream = null;

            propStream = new java.io.FileInputStream(user_properties_file);
            user_props = new java.util.Properties();
            user_props.load(propStream);
        }catch (Exception e) {
            // No user properties
            user_props = null;
        }
        
        // For each Storage type, generate an Hibernate xml file
        it = storage_types.iterator();
        
        while (it.hasNext())
        {
            st = (StorageTypeDecl) it.next();
            
            if (st != null)
            {
                // Prepare generation
                String package_name = translator_.getPackage(st);
                Set subclasses = get_user_subclasses(user_props, st);
                             
                create_file( base_dir,
                             package_name,
                             st.getName(),
                             "out" );
                             
                // Complete the file
                put("gen", this);
                put("translator", translator_);
                put("class", st);
                put("package_name", package_name);
                put("has_subclasses", new Boolean(has_user_subclasses(user_props, st)));
                put("subclasses", subclasses);
                map("HIBERNATE_FILE");
            }
        }
        
        // For each User class, generate an Hibernate xml file
// The following code is valid but we need to correct a bug in hibernate before use it.
// (Hibernate don't try to load the hbm.xml file of the super class)        
/*        for(Enumeration e = user_props.keys(); e.hasMoreElements(); )
        {
            String user_class = (String) e.nextElement();
            
            if (user_class != null)
            {
                create_file( base_dir,
                             getPackage(user_class),
                             getShortName(user_class),
                             "out" );
                             
                // Complete the file
                put("class", user_class);
                put("super_class", user_props.getProperty(user_class) );
                put("is_subclasses", new Boolean(true));
                map("HIBERNATE_FILE");
            }
        }
*/       
    }
}
